package com.jpsoft.smart.modules.base.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.json.JSONObject;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.jpsoft.smart.config.OSSConfig;
import com.jpsoft.smart.config.TemperatureConfig;
import com.jpsoft.smart.modules.base.dao.PersonDeviceFilterLogDAO;
import com.jpsoft.smart.modules.base.dao.PersonDeviceLogDAO;
import com.jpsoft.smart.modules.base.entity.*;
import com.jpsoft.smart.modules.base.service.*;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.modules.common.utils.LApiUtil;
import com.jpsoft.smart.modules.common.utils.OSSUtil;
import com.jpsoft.smart.modules.common.utils.TimeZonesUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import sun.misc.BASE64Decoder;

import javax.annotation.Resource;
import java.io.ByteArrayInputStream;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author 墨鱼_mo
 * @date 2020-3-20 9:14
 */
@Slf4j
@Transactional
@Component(value = "personDeviceFilterLogService")
public class PersonDeviceFilterLogServiceImpl implements PersonDeviceFilterLogService {
    @Autowired
    private OSSConfig ossConfig;

    @Autowired
    private PersonInfoService personInfoService;

    @Autowired
    private DeviceInfoService deviceInfoService;

    @Autowired
    private CheckNumCompanyService checkNumCompanyService;

    @Resource(name = "personDeviceFilterLogDAO")
    private PersonDeviceFilterLogDAO personDeviceFilterLogDAO;

    @Autowired
    private TemperatureConfig temperatureConfig;

    @Override
    public PersonDeviceFilterLog get(Long id) {
        // TODO Auto-generated method stub
        return personDeviceFilterLogDAO.get(id);
    }

    @Override
    @Async
    public void deviceInsertFilterLog(String deviceNo, BigDecimal temperature, JSONObject faceImageJson, JSONObject libMatInfoListJson, JSONObject matchPersonInfo, Date date,Integer maskFlag) {

        try {


            String retFileUrl = "";
            try {

                BASE64Decoder decoder = new BASE64Decoder();

                byte[] imgData = decoder.decodeBuffer(faceImageJson.getStr("Data"));

                for (int i = 0; i < imgData.length; ++i) {
                    if (imgData[i] < 0) {// 调整异常数据
                        imgData[i] += 256;
                    }
                }

                ByteArrayInputStream inputStream = new ByteArrayInputStream(imgData);


                retFileUrl = OSSUtil.upload(ossConfig, "/devicePersonLog", faceImageJson.getStr("Name"), inputStream);
            } catch (Exception e) {
                e.printStackTrace();
            }

            //记录的时间段
            String recordTimeZons = DateUtil.format(date, "yyyy-MM-dd-HH-mm");
            String timeZones = TimeZonesUtil.getUpperTimeZone(recordTimeZons);
            PersonInfo personInfo = LApiUtil.getPersonCode(libMatInfoListJson, matchPersonInfo);
            DeviceInfo deviceInfo = deviceInfoService.getByDeviceNo(deviceNo);

      //      PersonInfo personInfoReturn = personInfoService.addPersonForCompany(personInfo,deviceNo);
           // Integer personId = LApiUtil.getPersonCode(libMatInfoListJson, matchPersonInfo);
            LApiUtil lApiUtil = new LApiUtil();
            if (personInfo.getId() != 0L && temperature.compareTo(BigDecimal.ZERO) == 1) {


                PersonDeviceFilterLog personDeviceFilterLog = personDeviceFilterLogDAO.findByPersonIdAndDeviceNoAndTimeZones(Math.toIntExact(personInfo.getId()), deviceNo, timeZones);

                if (personDeviceFilterLog != null) {
                    BigDecimal oldTemper = personDeviceFilterLog.getTemperature();
                    personDeviceFilterLog.setUpdateTime(new Date());
                    personDeviceFilterLog.setTemperature(temperature);
                    personDeviceFilterLog.setRecordTime(date);
                    personDeviceFilterLog.setFaceImage(retFileUrl);
                    personDeviceFilterLog.setCreateTime(new Date());

                    if (StringUtils.isNotBlank(libMatInfoListJson.getStr("MatchStatus"))) {
                        personDeviceFilterLog.setMatchStatus(libMatInfoListJson.getInt("MatchStatus"));
                        personDeviceFilterLog.setMatchMsg(lApiUtil.getMatchMsg(libMatInfoListJson.getInt("MatchStatus"),maskFlag,temperature,temperatureConfig));
                        personDeviceFilterLog.setMatchFaceId(libMatInfoListJson.getInt("MatchFaceID"));
                    } else {
                        personDeviceFilterLog.setMatchStatus(0);
                        personDeviceFilterLog.setMatchFaceId(0);
                    }

                    if (!LApiUtil.compareToInterval(oldTemper, temperatureConfig.getMin(), temperatureConfig.getMax())) {
                        if (LApiUtil.compareToInterval(temperature, temperatureConfig.getMin(), temperatureConfig.getMax())) {

                            personDeviceFilterLogDAO.update(personDeviceFilterLog);
                        } else {
                            if (temperature.compareTo(oldTemper) == -1) {
                                personDeviceFilterLogDAO.update(personDeviceFilterLog);
                            }
                        }
                    }
                } else {
                    PersonDeviceFilterLog personDeviceFilterLog1 = new PersonDeviceFilterLog();
                    personDeviceFilterLog1.setDeviceNo(deviceNo);
                    personDeviceFilterLog1.setPersonId(Math.toIntExact(personInfo.getId()));
                    personDeviceFilterLog1.setTemperature(temperature);
                    if (StringUtils.isNotBlank(libMatInfoListJson.getStr("MatchStatus"))) {
                        personDeviceFilterLog1.setMatchStatus(libMatInfoListJson.getInt("MatchStatus"));
                        personDeviceFilterLog1.setMatchMsg(lApiUtil.getMatchMsg(libMatInfoListJson.getInt("MatchStatus"),maskFlag,temperature,temperatureConfig));
                        personDeviceFilterLog1.setMatchFaceId(libMatInfoListJson.getInt("MatchFaceID"));
                    } else {
                        personDeviceFilterLog1.setMatchStatus(0);
                        personDeviceFilterLog1.setMatchFaceId(0);
                    }
                    personDeviceFilterLog1.setFaceImage(retFileUrl);
                    personDeviceFilterLog1.setRecordTime(date);
                    personDeviceFilterLog1.setTimeZones(timeZones);
                    personDeviceFilterLog1.setDelFlag(false);
                    personDeviceFilterLog1.setCreateTime(new Date());

                    personDeviceFilterLogDAO.insert(personDeviceFilterLog1);
                }

                new Thread(() -> {
                    try {
                        addCheckNumForCompany(Math.toIntExact(personInfo.getId()), temperature, date);
                    } catch (Exception e) {
                        log.error(e.getMessage(), e);
                    }
                }).start();
            }
            if (personInfo.getId() == 0L && temperature.compareTo(BigDecimal.valueOf(temperatureConfig.getMax())) == 1){
                addVisitorCheckNumForCompany(deviceInfo.getCompanyId(), temperature, date);
            }

        } catch (Exception e) {

            log.error(e.getMessage());
        }


    }

    public void addCheckNumForCompany(Integer personId, BigDecimal temperature, Date date) {

        String recordTime = DateUtil.format(date, "yyyy-MM-dd");
        PersonInfo personInfo = personInfoService.get(personId.longValue());
        if (StringUtils.isNotBlank(personInfo.getCompanyId())) {
            CheckNumCompany checkNumCompany = checkNumCompanyService.findByCompanyIdAndRecordTime(personInfo.getCompanyId(), recordTime);
            if (checkNumCompany != null) {
                Integer checkNum = checkNumCompany.getCheckNum().intValue() + 1;
                checkNumCompany.setCheckNum(checkNum.longValue());
                checkNumCompanyService.update(checkNumCompany);
            } else {
                CheckNumCompany checkNumCompany1 = new CheckNumCompany();
                checkNumCompany1.setCompanyId(personInfo.getCompanyId());
                checkNumCompany1.setCheckNum(1L);
                checkNumCompany1.setDelFlag(false);
                checkNumCompany1.setRecordTime(recordTime);
                checkNumCompany1.setCreateTime(new Date());
                checkNumCompanyService.insert(checkNumCompany1);
            }
        }
    }

    public void addVisitorCheckNumForCompany(String companyId, BigDecimal temperature, Date date) {

        String recordTime = DateUtil.format(date, "yyyy-MM-dd");

        if (StringUtils.isNotBlank(companyId)) {
            CheckNumCompany checkNumCompany = checkNumCompanyService.findByCompanyIdAndRecordTime(companyId, recordTime);
            if (checkNumCompany != null) {
                Integer checkNum = checkNumCompany.getCheckNum().intValue() + 1;
                checkNumCompany.setCheckNum(checkNum.longValue());
                checkNumCompanyService.update(checkNumCompany);
            } else {
                CheckNumCompany checkNumCompany1 = new CheckNumCompany();
                checkNumCompany1.setCompanyId(companyId);
                checkNumCompany1.setCheckNum(1L);
                checkNumCompany1.setDelFlag(false);
                checkNumCompany1.setRecordTime(recordTime);
                checkNumCompany1.setCreateTime(new Date());
                checkNumCompanyService.insert(checkNumCompany1);
            }
        }
    }

    @Override
    public Page<PersonDeviceFilterLog> pageSearch(Map<String, Object> searchParams, int pageNumber, int pageSize, boolean count, List<Sort> sortList) {
        Page<PersonDeviceFilterLog> page = PageHelper.startPage(pageNumber, pageSize, count).doSelectPage(() -> {
            personDeviceFilterLogDAO.search(searchParams, sortList);
        });

        return page;
    }

    @Override
    public List<PersonDeviceFilterLog> findByPersonAndDate(Long personId, String startDate, String endDate) {
        return personDeviceFilterLogDAO.findByPersonAndDate(personId, startDate, endDate);
    }

    @Override
    public PersonDeviceFilterLog findByPersonOrderTemperature(Long personId, String startDate, String endDate) {
        return personDeviceFilterLogDAO.findByPersonOrderTemperature(personId, startDate, endDate);
    }

    @Override
    public PersonDeviceFilterLog lastPersonLog(Long personId) {
        return personDeviceFilterLogDAO.lastPersonLog(personId);
    }

    @Override
    public PersonDeviceFilterLog getByPersonIdAndDate(Long personId, String startDate, String endDate) {
        return personDeviceFilterLogDAO.getByPersonIdAndDate(personId, startDate, endDate);
    }


    @Override
    public List<PersonDeviceFilterLog> list() {
        return personDeviceFilterLogDAO.list();
    }

    @Override
    public long countByTimeRangeAndCompanyCode(String companyCode, Long personId, int status,float temperatureMax, Date startDate, Date endDate) {
        return personDeviceFilterLogDAO.countByTimeRangeAndCompanyCode(companyCode, personId, status,temperatureMax, startDate, endDate);
    }

    @Override
    public List<PersonDeviceFilterLog> temperatureNormalStatistics(String companyCode, BigDecimal temperature, Date startDate, Date endDate) {

        return personDeviceFilterLogDAO.temperatureNormalStatistics(companyCode, temperature, startDate, endDate);
    }

    @Override
    public List<PersonDeviceFilterLog> temperatureUnusualStatistics(String companyCode, BigDecimal temperature, Date startDate, Date endDate) {
        return personDeviceFilterLogDAO.temperatureUnusualStatistics(companyCode, temperature, startDate, endDate);
    }

    @Override
    public List<Map<String, Object>> unusualStatistics(String companyCode, Long personId, BigDecimal temperature, Date startDate, Date endDate) {
        return personDeviceFilterLogDAO.unusualStatistics(companyCode, personId, temperature, startDate, endDate);
    }

    @Override
    public long countUndetectedByCompanyCode(String companyCode, List<PersonCompany> personCompanyList, Date startTime, Date endTime) {
        return personDeviceFilterLogDAO.countUndetectedByCompanyCode(companyCode, personCompanyList, startTime, endTime);
    }

    @Override
    public Page<PersonInfo> findUndetectedPersonList(Map<String,Object> searchParams,int pageNumber,int pageSize,boolean count) {
        Page<PersonInfo> page = PageHelper.startPage(pageNumber,pageSize,count).doSelectPage(()->{
            personDeviceFilterLogDAO.findUndetectedByCompanyCode(searchParams);
        });

        return page;
    }

    @Override
    public Integer getDayCheckedPersonNumByCompanyList(Date startTime, Date endTime, List<CompanyInfo> list,String popedom) {
        return personDeviceFilterLogDAO.getDayCheckedPersonNumByCompanyList(startTime,endTime,list,popedom);
    }

    @Override
    public Integer getDayAbnormalNum(Date startTime, Date endTime) {
        return personDeviceFilterLogDAO.getDayAbnormalNum(startTime,endTime,temperatureConfig.getMax());
    }

    @Override
    public Integer getDayAbnormalNumByCompanyList(Date startTime, Date endTime, List<CompanyInfo> list) {
        Integer total = personDeviceFilterLogDAO.getDayAbnormalNumByCompanyList(startTime,endTime,list,temperatureConfig.getMax());
       if (total == null){
           return 0;
       }else {
           return total;
       }

    }

    @Override
    public Integer getDayNormalNumByCompanyId(Date startTime,Date endTime, String companyId) {
        return personDeviceFilterLogDAO.getDayNormalNumByCompanyId(startTime,endTime,companyId,temperatureConfig.getMax());
    }

    @Override
    public Integer countDayByPersonIdAndDate(Long personId, Date startDate, Date endDate) {
        return personDeviceFilterLogDAO.countDayByPersonIdAndDate(personId,startDate,endDate);
    }

    @Override
    public Integer countDetectedByCompanyId(String companyId, Date startDate, Date endDate) {
        return personDeviceFilterLogDAO.countDetectedByCompanyId(companyId,startDate,endDate);
    }

    @Override
    public int deleteExpiredVisitorRecord(int limit) {
        return personDeviceFilterLogDAO.deleteExpiredVisitorRecord(limit);
    }

    @Override
    public List<PersonDeviceFilterLog> getDayCheckedPersonByCompanyList(Date beginTime, Date endTime, CompanyInfo companyInfo) {
        return personDeviceFilterLogDAO.getDayCheckedPersonByCompanyList(beginTime,endTime,companyInfo);
    }

    @Override
    public long countDetectedByCompanyIds(List<String> companyIds, Date startTime, Date endTime) {
        return personDeviceFilterLogDAO.countDetectedByCompanyIds(companyIds,  startTime, endTime);
    }

    @Override
    public Page<PersonInfo> findUndetectedPersonListByCompanyIds(Map<String, Object> searchParams, int pageIndex, int pageSize, boolean count) {
        Page<PersonInfo> page = PageHelper.startPage(pageIndex,pageSize,count).doSelectPage(()->{
            personDeviceFilterLogDAO.findUndetectedPersonListByCompanyIds(searchParams);
        });

        return page;
    }
}
