package com.jpsoft.smart.modules.base.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.json.JSONObject;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.jpsoft.smart.config.OSSConfig;
import com.jpsoft.smart.config.TemperatureConfig;
import com.jpsoft.smart.config.WxConfig;
import com.jpsoft.smart.modules.base.dao.*;
import com.jpsoft.smart.modules.base.entity.*;
import com.jpsoft.smart.modules.base.service.PersonDeviceLogService;
import com.jpsoft.smart.modules.base.service.PersonInfoService;
import com.jpsoft.smart.modules.base.service.WarningPusherService;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.modules.common.utils.*;
import com.jpsoft.smart.modules.common.utils.Base64;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import sun.misc.BASE64Decoder;

import javax.annotation.Resource;
import javax.xml.crypto.Data;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;

/**
 * @author 墨鱼_mo
 * @date 2020-3-15 14:12
 */
@Slf4j
@Transactional
@Component(value = "personDeviceLogService")
public class PersonDeviceLogServiceImpl implements PersonDeviceLogService {

    @Autowired
    private OSSConfig ossConfig;

    @Autowired
    private WxConfig wxConfig;

    @Autowired
    private TemperatureConfig temperatureConfig;

    @Autowired
    private WarningPusherService warningPusherService;

    @Autowired
    private PersonInfoService personInfoService;

    @Resource(name = "personDeviceLogDAO")
    private PersonDeviceLogDAO personDeviceLogDAO;

    @Resource(name = "personInfoDAO")
    private PersonInfoDAO personInfoDAO;

    @Resource(name = "companyInfoDAO")
    private CompanyInfoDAO companyInfoDAO;

    @Resource(name = "deviceInfoDAO")
    private DeviceInfoDAO deviceInfoDAO;


    @Override
    @Async
    public void deviceInsertLog(String deviceNo, BigDecimal temperature, JSONObject faceImageJson, JSONObject libMatInfoListJson, JSONObject matchPersonInfo, Date date,Integer maskFlag) {
        String retFileUrl = "";
        try {

            if (StringUtils.isNotBlank(faceImageJson.getStr("Data"))){
                BASE64Decoder decoder = new BASE64Decoder();

                byte[] imgData = decoder.decodeBuffer(faceImageJson.getStr("Data"));

                for (int i = 0; i < imgData.length; ++i) {
                    if (imgData[i] < 0) {// 调整异常数据
                        imgData[i] += 256;
                    }
                }

                ByteArrayInputStream inputStream = new ByteArrayInputStream(imgData);


                retFileUrl = OSSUtil.upload(ossConfig, "/devicePersonLog", faceImageJson.getStr("Name"), inputStream);
            }

        } catch (Exception e) {
            log.error("保存图片路径错误" + e.getMessage());
        }

        PersonInfo personInfo = LApiUtil.getPersonCode(libMatInfoListJson, matchPersonInfo);
      //  PersonInfo peronInfoReturn = personInfoService.addPersonForCompany(personInfo,deviceNo);
      //  Integer personId = LApiUtil.getPersonCode(libMatInfoListJson, matchPersonInfo);

        PersonDeviceLog personDeviceLog = new PersonDeviceLog();
        personDeviceLog.setId(UUID.randomUUID().toString());
        personDeviceLog.setDeviceNo(deviceNo);
        personDeviceLog.setPersonId(Math.toIntExact(personInfo.getId()));
        personDeviceLog.setPersonName(personInfo.getName());
        personDeviceLog.setIdentityNo(personInfo.getIdCard());

        personDeviceLog.setTemperature(temperature);
        LApiUtil lApiUtil = new LApiUtil();
        if (StringUtils.isNotBlank(libMatInfoListJson.getStr("MatchStatus"))) {
            personDeviceLog.setMatchStatus(libMatInfoListJson.getInt("MatchStatus"));
            personDeviceLog.setMatchMsg(lApiUtil.getMatchMsg(libMatInfoListJson.getInt("MatchStatus"),maskFlag,temperature,temperatureConfig));
            personDeviceLog.setMatchFaceId(libMatInfoListJson.getInt("MatchFaceID"));
        } else {
            personDeviceLog.setMatchStatus(0);
            personDeviceLog.setMatchFaceId(0);
        }

        personDeviceLog.setRecordTime(date);
        personDeviceLog.setDelFlag(false);
        personDeviceLog.setCreateTime(new Date());
        personDeviceLog.setFaceImage(retFileUrl);
        personDeviceLogDAO.insert(personDeviceLog);

        //人员识别成功，温度超过限定时发送模板消息给监管者
        new Thread(()->{
            try {
                if (personInfo.getId() != 0L) {
                    sendTemperatureAlarmsyncTask(personDeviceLog);
                }
            }
            catch (Exception ex){
                log.error(ex.getMessage(),ex);
            }
        }).start();



    }


    public void sendTemperatureAlarmsyncTask(PersonDeviceLog personDeviceLog) {

        try {

            PersonInfo personInfo = personInfoDAO.get(personDeviceLog.getPersonId().longValue());
            if (personInfo == null) {
                throw new Exception(personDeviceLog.getPersonId() + "对应的人员不存在");
            }
            CompanyInfo companyInfo = companyInfoDAO.get(personInfo.getCompanyId());
            if (companyInfo == null) {
                throw new Exception(personInfo.getName() + "对应的公司不存在");
            }
            DeviceInfo deviceInfo = deviceInfoDAO.findByDeviceNo(personDeviceLog.getDeviceNo());
            if (deviceInfo == null) {
                throw new Exception(personDeviceLog.getDeviceNo() + "设备不存在");
            }

            List<String> idList = Arrays.asList(companyInfo.getCode().split(","));

            List<WarningPusher> warningPusherList = warningPusherService.findByCompanyIdList(idList);
            if (warningPusherList.size() > 0) {

                for (WarningPusher warningPusher : warningPusherList) {
                    Map<String, Object> searchParams = new HashMap<>();
                    searchParams.put("deviceNo", personDeviceLog.getDeviceNo());
                    searchParams.put("personId", personDeviceLog.getPersonId());
                    searchParams.put("minTemperature", warningPusher.getTemperature());
                    searchParams.put("beginTime", new Date(personDeviceLog.getRecordTime().getTime() - 300000));
                    List<Sort> sortList = new ArrayList<>();
                    sortList.add(new Sort("record_time", "desc"));
                    List<PersonDeviceLog> personDeviceLogList = personDeviceLogDAO.search(searchParams, sortList);

                    if (personDeviceLogList.size() >= warningPusher.getTimes()) {
                        WechatMessageUtil.sendTemperatureAlarmInfo(warningPusher.getOpenId(), companyInfo.getName() + "_" + deviceInfo.getAliasName(), wxConfig.getAppId(), wxConfig.getAppSecret(), personDeviceLog.getId());
                    }
                }
            }

        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }

    }



    @Override
    public Page<PersonDeviceLog> pageSearch(Map<String, Object> searchParams, int pageNumber, int pageSize, boolean count, List<Sort> sortList) {
        Page<PersonDeviceLog> page = PageHelper.startPage(pageNumber, pageSize, count).doSelectPage(() -> {
            personDeviceLogDAO.search(searchParams, sortList);
        });

        return page;
    }


    @Override
    public Page<PersonDeviceLog> simplePageSearch(Map<String, Object> searchParams, int pageNumber, int pageSize, boolean count, List<Sort> sortList) {
        Page<PersonDeviceLog> page = PageHelper.startPage(pageNumber, pageSize, count).doSelectPage(() -> {
            personDeviceLogDAO.simpleSearch(searchParams, sortList);
        });

        return page;
    }

    @Override
    public PersonDeviceLog get(String id) {
        return personDeviceLogDAO.get(id);
    }

    @Override
    public List<PersonDeviceLog> findByPersonAndDate(Long personId, Date startDate, Date endDate) {
        return personDeviceLogDAO.findByPersonAndDate(personId, startDate, endDate);
    }


    @Override
    public List<PersonInfo> queryUnAttendanceList(String companyCode, Date startTime, Date endTime) {
        return personDeviceLogDAO.queryUnAttendanceList(companyCode, startTime, endTime);
    }

    @Override
    public List<PersonInfo> queryAttendanceList(String companyCode, Date startTime, Date endTime) {
        return personDeviceLogDAO.queryAttendanceList(companyCode, startTime, endTime);
    }

    @Override
    public long countByAttendance(String companyCode, Date beginDate, Date endDate) {
        return personDeviceLogDAO.countByAttendance(companyCode,beginDate,endDate);
    }

    @Override
    public Integer getDayAbnormalNumByDeviceNoList(DateTime dateTime, Date date, List<String> deviceNoList) {
        Integer total = personDeviceLogDAO.getDayAbnormalNumByDeviceNoList(dateTime,date,deviceNoList,temperatureConfig.getMax());
        if (total == null){
            return 0;
        }else{
            return total;
        }


    }

    @Override
    public PersonDeviceLog findLastPersonLog(Long personId, float temperatureMax) {
        return personDeviceLogDAO.findLastPersonLog(personId,temperatureMax);
    }

    @Override
    public Integer getDayCheckedVistorNumByDeviceNoList(Date startTime, Date endTime, List<String> deviceNoList) {
        return personDeviceLogDAO.getDayCheckedVistorNumByDeviceNoList(startTime,endTime,deviceNoList);
    }

    @Override
    public Page<PersonDeviceLog> findLastPersonLogByCompanyList(DateTime beginOfDay, DateTime endOfDay, List<CompanyInfo> list,int pageNum, int pageSize) {

        Page<PersonDeviceLog> page = PageHelper.startPage(pageNum, pageSize).doSelectPage(() -> {
            personDeviceLogDAO.findLastPersonLogByCompanyList(beginOfDay,endOfDay,list);
        });

        return page;
    }

    @Override
    public Page<PersonDeviceLog> findVisitorLogByDeviceNoList(DateTime beginOfDay, DateTime endOfDay, List<String> deviceNoList, int pageIndex, int pageSize) {
        Page<PersonDeviceLog> page = PageHelper.startPage(pageIndex, pageSize).doSelectPage(() -> {
            personDeviceLogDAO.findVisitorLogByDeviceNoList(beginOfDay,endOfDay,deviceNoList);
        });

        return page;
    }

    @Override
    public Page<PersonDeviceLog> findAllPersonLogByCompanyListAndDeviceNoList(DateTime beginOfDay, DateTime endOfDay, List<CompanyInfo> companyList,List<String> deviceNoList,int pageIndex, int pageSize) {
        Page<PersonDeviceLog> page = PageHelper.startPage(pageIndex, pageSize).doSelectPage(() -> {
            personDeviceLogDAO.findAllPersonLogByCompanyIdAndDeviceNo(beginOfDay,endOfDay,companyList,deviceNoList);
        });
        return page;
    }

    @Override
    public Page<PersonDeviceLog> findLastTeacherLogByCompanyList(Date beginOfDay, Date endOfDay, List<CompanyInfo> list, int pageIndex, int pageSize) {
        Page<PersonDeviceLog> page = PageHelper.startPage(pageIndex, pageSize).doSelectPage(() -> {
            personDeviceLogDAO.findLastTeacherLogByCompanyList(beginOfDay,endOfDay,list);
        });

        return page;
    }

    @Override
    public Page<PersonDeviceLog> findLastStudentLogByCompanyList(Date beginOfDay, Date endOfDay, List<CompanyInfo> list, int pageIndex, int pageSize) {
        Page<PersonDeviceLog> page = PageHelper.startPage(pageIndex, pageSize).doSelectPage(() -> {
            personDeviceLogDAO.findLastStudentLogByCompanyList(beginOfDay,endOfDay,list);
        });

        return page;
    }

    @Override
    public Integer getDayCheckedPersonTotalByCompanyList(Date startTime, Date endTime, List<CompanyInfo> list, String popedom) {
        Integer total = personDeviceLogDAO.getDayCheckedPersonNumByCompanyList(startTime, endTime, list, popedom);

        if (total!=null){
            return total;
        }
        else{
            return 0;
        }
    }

    @Override
    public List<PersonDeviceLog> findExpiredVisitorRecord(int limit) {
        return personDeviceLogDAO.findExpiredVisitorRecord(limit);
    }

    @Override
    public int delete(String id) {
        return personDeviceLogDAO.delete(id);
    }
}
