package com.jpsoft.smart.modules.business.service.impl;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import javax.annotation.Resource;

import com.jpsoft.smart.modules.base.dao.AlarmConfigDAO;
import com.jpsoft.smart.modules.base.dao.CompanyInfoDAO;
import com.jpsoft.smart.modules.base.dao.PersonInfoDAO;
import com.jpsoft.smart.modules.base.entity.AlarmConfig;
import com.jpsoft.smart.modules.base.entity.CompanyInfo;
import com.jpsoft.smart.modules.base.entity.PersonInfo;
import com.jpsoft.smart.modules.base.service.HolidayInfoService;
import com.jpsoft.smart.modules.common.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.weaver.patterns.PerSingleton;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import com.jpsoft.smart.modules.business.dao.WorkAttendanceDAO;
import com.jpsoft.smart.modules.business.entity.WorkAttendance;
import com.jpsoft.smart.modules.business.service.WorkAttendanceService;
import com.github.pagehelper.Page;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.github.pagehelper.PageHelper;

@Slf4j
@Transactional
@Component(value="workAttendanceService")
public class WorkAttendanceServiceImpl implements WorkAttendanceService {
	@Resource(name="workAttendanceDAO")
	private WorkAttendanceDAO workAttendanceDAO;

	@Autowired
	private PersonInfoDAO personInfoDAO;

	@Autowired
	private CompanyInfoDAO companyInfoDAO;

	@Autowired
	private AlarmConfigDAO alarmConfigDAO;

	@Autowired
	private HolidayInfoService holidayInfoService;

	@Override
	public WorkAttendance get(String id) {
		// TODO Auto-generated method stub
		return workAttendanceDAO.get(id);
	}

	@Override
	public int insert(WorkAttendance model) {
		// TODO Auto-generated method stub
		//model.setId(UUID.randomUUID().toString());
		
		return workAttendanceDAO.insert(model);
	}

	@Override
	public int update(WorkAttendance model) {
		// TODO Auto-generated method stub
		return workAttendanceDAO.update(model);		
	}

	@Override
	public int delete(String id) {
		// TODO Auto-generated method stub
		return workAttendanceDAO.delete(id);
	}

	@Override
	public boolean exist(String id) {
		// TODO Auto-generated method stub
		int count = workAttendanceDAO.exist(id);
		
		return count > 0 ? true : false;
	}
	
	@Override
	public List<WorkAttendance> list() {
		// TODO Auto-generated method stub
		return workAttendanceDAO.list();
	}
		
	@Override
	public Page<WorkAttendance> pageSearch(Map<String, Object> searchParams, int pageNumber, int pageSize,boolean count,List<Sort> sortList) {
        Page<WorkAttendance> page = PageHelper.startPage(pageNumber,pageSize,count).doSelectPage(()->{
            workAttendanceDAO.search(searchParams,sortList);
        });
        
        return page;
	}

	@Override
	public List<WorkAttendance> findByPersonIdAndDate(Long personId, Date startDate, Date endDate) {
		return workAttendanceDAO.findByPersonIdAndDate(personId,startDate,endDate);
	}

	@Override
	public int deleteByCompanyCodeAndDate(String companyCode, Date startDate, Date endDate) {
		return workAttendanceDAO.deleteByCompanyCodeAndDate(companyCode,startDate,endDate);
	}

	@Override
	public boolean punchIn(Long personId, BigDecimal temperature, Date recordDate) {
		boolean result = false;

		try {
			DateTime recordTime = new DateTime(recordDate);
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");
			String date = recordTime.toString("yyyy-MM-dd");

			//是否节假日
			Set<String> holidaySet = holidayInfoService.getHolidaySetByDate(recordDate,recordDate);

			if (holidaySet.contains(recordDate)){
				return false;
			}

			PersonInfo personInfo = personInfoDAO.get(personId);
			CompanyInfo companyInfo = companyInfoDAO.get(personInfo.getCompanyId());

			String[] arr = companyInfo.getCode().split(",");

			List<AlarmConfig> configList = null;

			for (int i = arr.length - 1; i >= 0; i--) {
				configList = alarmConfigDAO.findByCompanyId(arr[i]);

				if (configList != null && configList.size() > 0) {
					break;
				}
			}

			if (configList != null) {
				for (AlarmConfig alarmConfig : configList) {
					//是否工作日
					if (alarmConfig.getWeekdays().indexOf(String.valueOf(recordTime.getDayOfWeek())) == -1) {
						continue;
					}

					DateTime startTime = new DateTime(sdf.parse(date + " " + alarmConfig.getStartTime()));
					DateTime attendanceTime = new DateTime(sdf.parse(date + " " + alarmConfig.getAttendanceTime()));
					DateTime endTime = new DateTime(sdf.parse(date + " " + alarmConfig.getEndTime()));

					String attendanceResult = "";

					if (alarmConfig.getClassifier().equals(1)) {
						//todo 上班考勤

						//准时上班
						if (recordTime.compareTo(startTime)>0 && recordTime.compareTo(attendanceTime)<=0){
							attendanceResult = WorkAttendance.SUCCESS;
						}
						else if(recordTime.compareTo(attendanceTime)>0 && recordTime.compareTo(endTime)<=0){
							attendanceResult = WorkAttendance.LATE;
						}

						if(StringUtils.isNotEmpty(attendanceResult)){
							//当天是否已经有该时段上班打卡记录，如果有则不记录
							List<WorkAttendance> workAttendances = workAttendanceDAO.findByPeriod(personId,alarmConfig.getId(),recordTime.toDate());

							if (workAttendances.size()>0){
								continue;
							}
						}
					} else {
						//todo 下班考勤

						//早退
						if (recordTime.compareTo(startTime)>0 && recordTime.compareTo(attendanceTime)<=0){
							attendanceResult = WorkAttendance.LEAVE_EARLY;
						}
						else if(recordTime.compareTo(attendanceTime)>0 && recordTime.compareTo(endTime)<=0){
							attendanceResult = WorkAttendance.SUCCESS;
						}

						if(StringUtils.isNotEmpty(attendanceResult)) {
							//当天是否已经有该时段记录,有则删除
							List<WorkAttendance> workAttendances = workAttendanceDAO.findByPeriod(personId, alarmConfig.getId(), recordTime.toDate());

							for (WorkAttendance workAttendance : workAttendances) {
								workAttendance.setDelFlag(true);
								workAttendance.setUpdateTime(new Date());

								workAttendanceDAO.update(workAttendance);
							}
						}
					}

					if (StringUtils.isNotEmpty(attendanceResult)) {
						WorkAttendance workAttendance = new WorkAttendance();
						workAttendance.setId(UUID.randomUUID().toString());
						workAttendance.setPersonId(personInfo.getId());
						workAttendance.setAlarmConfigId(alarmConfig.getId());
						workAttendance.setCreateTime(new Date());
						workAttendance.setDelFlag(false);

						workAttendance.setTemperature(temperature);
						workAttendance.setResult(attendanceResult);
						workAttendance.setRecordTime(recordDate);

						int affectCount = workAttendanceDAO.insert(workAttendance);

						result = affectCount > 0;

						break;
					}
				}
			}
		}
		catch (Exception ex){
			result = false;
			log.error(ex.getMessage(),ex);
		}

		return result;
	}

	@Override
	public List<PersonInfo> queryUnAttendanceList(String companyCode, Date startDate, Date endDate) {
		return workAttendanceDAO.queryUnAttendanceList(companyCode,startDate,endDate);
	}
}