package com.jpsoft.smart.modules.lapi.request;

import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSONObject;
import com.jpsoft.smart.modules.lapi.channel.LapiChannel;
import com.jpsoft.smart.modules.sys.entity.SysLog;
import com.jpsoft.smart.modules.sys.service.SysLogService;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.handler.codec.http.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.UnsupportedEncodingException;
import java.net.SocketAddress;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.TimeUnit;

@Slf4j
@Component
public class LapiRequest {
    @Autowired
    private Map<String, LapiChannel> activeLapiChannelMap;

    @Autowired
    private SysLogService sysLogService;

    public JSONObject send(String deviceNo, String uri,HttpMethod httpMethod,HashMap map) throws Exception{
        LapiChannel lapiChannel = activeLapiChannelMap.get(deviceNo);

        if (lapiChannel==null){
            log.error(deviceNo + "不在线!");
            throw new Exception("设备不在线");
     //       return null;
        }

        String msg = "";

        if (map!=null) {
            msg = new JSONObject(map).toJSONString();
        }

        ByteBuf byteBuf = Unpooled.wrappedBuffer(msg.getBytes("UTF-8"));

        FullHttpRequest request = new DefaultFullHttpRequest(HttpVersion.HTTP_1_1, httpMethod,uri,byteBuf);

        //必须设置content_length,否则返回报错
        request.headers().set(HttpHeaderNames.CONTENT_LENGTH, request.content().readableBytes());

        lapiChannel.getChannel().writeAndFlush(request).sync();

        SysLog sysLog = new SysLog();
        sysLog.setPointcut(deviceNo);
        sysLog.setUrl(uri);

        BlockingQueue<String> queue = lapiChannel.getReceivePacketQueue(uri);

        String body = "";

        if(queue!=null) {
            body = queue.poll(10, TimeUnit.SECONDS);
        }
        else{
            sysLog.setRemark("设备" + deviceNo + "uri对应队列不存在!");
        }

        if (StringUtils.isNotEmpty(msg) && msg.length()>255) {
            sysLog.setData(msg.substring(0,255));
        }
        else{
            sysLog.setData(msg);
        }

        if (StringUtils.isNotEmpty(body)){
            if (body.length()>255){
                sysLog.setRemark(body.substring(0,255));
            }
            else{
                sysLog.setRemark(body);
            }
        }

        SocketAddress remoteAddress = lapiChannel.getChannel().remoteAddress();
        sysLog.setRemoteIp(remoteAddress.toString());
        sysLog.setCreateTime(new Date());
        sysLogService.insert(sysLog);

        if (StringUtils.isNotEmpty(body)){
            JSONObject jsonObject = JSONObject.parseObject(body);

            return jsonObject;
        }
        else{
            throw new Exception("设备不在线");
        //    return null;
        }
    }
}
