package com.jpsoft.smart.modules.lapi.service;

import com.jpsoft.smart.modules.base.entity.DeviceInfo;
import com.jpsoft.smart.modules.base.entity.PersonInfo;
import com.jpsoft.smart.modules.lapi.dto.DataServersDTO;
import com.jpsoft.smart.modules.lapi.dto.ServerAddrDTO;
import com.jpsoft.smart.modules.lapi.vo.LapiMsgResult;
import com.jpsoft.smart.modules.lapi.vo.LapiResult;
import com.sun.org.apache.xpath.internal.operations.Bool;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * @author 墨鱼_mo
 * @date 2020-3-15 9:36
 */
public interface ILapiService {

    /**
     * 通过ip和port查看门禁设备，有设备信息返回为正常
     * @param ipAddress
     * @param port
     * @return
     * @throws Exception
     */
     DeviceInfo findByIpAddressAndPort(String ipAddress, String port) throws Exception;

    /**
     * 根据设备id查看设备在线状态
     * @param id 设备id
     * @return
     */
     Boolean keepAlive(String id) throws Exception;


    /**
     * 平台上传设备人脸信息
     * @param id
     * @return
     * @throws Exception
     */
     List<LapiResult> addPerson(Long id) throws Exception;


    /**
     * 人脸数据根据设备id上传
     * @param id
     * @param deviceIds
     * @return
     * @throws Exception
     */
     List<LapiResult> addPersonForDeviceIds(Long id ,String deviceIds) throws Exception;

    /**
     * 根据公司名称获取人脸库
     * @param companyName
     * @return
     */
     String getFaceDbId(DeviceInfo deviceInfo, String companyName) throws Exception;

    /**
     * 删除设备人脸数据
     * @param id
     * @return
     */
    List<LapiMsgResult> deletePerson(Long id) throws Exception;


    /**
     * 根据设备ID和人员id删除数据
     * @param id 人员id
     * @param deviceIds 设备id（多个以逗号隔开）
     * @return
     * @throws Exception
     */
    List<LapiMsgResult> deleteDevicePerson(Long id,String deviceIds) throws Exception;

    /**
     * 远程手机开门
     * @param deviceInfoId
     * @return
     * @throws Exception
     */
    Boolean phoneOpenDoor(String deviceInfoId) throws Exception;


    /**
     * 重启设备
     * @param deviceInfoId 设备id
     * @return
     */
    Boolean reBoot(String deviceInfoId) throws Exception;


    /**
     * 添加公司照片库
     * @param companyInfoId 公司id
     * @param deviceInfoIds 设备id
     * @return
     * @throws Exception
     */
    List<LapiResult> addCompanyLib(String companyInfoId, String deviceInfoIds) throws Exception;


    /**
     * 删除公司人脸库
     * @param deviceInfoId 设备id
     * @param libraryId 人脸库id
     * @return
     * @throws Exception
     */
    LapiResult deleteCompanyLib(String deviceInfoId,String libraryId) throws Exception;


    /**
     * 更新公司人脸库名称
     * @param companyId  公司id
     * @param deviceInfoId
     * @param libraryId
     * @return
     * @throws Exception
     */
    LapiResult updateCompanyLib(String companyId,String deviceInfoId,String libraryId) throws Exception;

    /**
     * 删除设备上的所有人脸库
     * @param deviceInfoId
     * @return
     * @throws Exception
     */
    List<LapiResult> deleteDeviceAllCompanyLib(String deviceInfoId) throws Exception;


    /**
     * 人员是否成功写入设备
     * @param
     * @return
     */
    Boolean isSuccessAddPerson(PersonInfo personInfo, DeviceInfo deviceInfo, String faceDbId);

    /**
     * 人员照片是否存在设备
     * @param
     * @return
     */
    Boolean isPersonImageExist(PersonInfo personInfo, DeviceInfo deviceInfo, String faceDbId);

    /**
     * 获取设备的验证规则
     * @param deviceInfoId
     * @return
     * @throws Exception
     */
    Map<String,Object> getDeviceRule(String deviceInfoId) throws Exception;


    /**
     * 设置设备的验证规则
     * @param deviceInfoId 设备id
     * @param needCap 安全帽验证 （0：不验证，1：验证）
     * @param needMask 口罩验证 （0：不验证，1：验证）
     * @param measureTemperature 测温验证 （0：不验证，1：验证）
     * @param type 测试验证类型 （0：测额温，1：测腕温）
     * @param minTemper 温度下限
     * @param maxTemper 温度上限
     * @param alarmThreshold 报警温度
     * @return
     * @throws Exception
     */
    Boolean setDeviceRule(String deviceInfoId, Integer needCap, Integer needMask, Integer measureTemperature, Integer type, BigDecimal minTemper,BigDecimal maxTemper,BigDecimal alarmThreshold) throws Exception;

    /**
     * 修改设备的设备序列号
     * @param oldDeviceNo
     * @param newDeviceNo
     * @return
     */
    Boolean updateDeviceNo(String oldDeviceNo,String newDeviceNo) throws Exception;

    /**
     * 获取设备的服务器信息
     * @param deviceNo
     * @return
     */
    DataServersDTO getDeviceDateServer(String deviceNo) throws Exception;


}
