package com.jpsoft.smart.modules.lapi.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.gson.JsonObject;
import com.jpsoft.smart.modules.base.entity.CompanyInfo;
import com.jpsoft.smart.modules.base.entity.DeviceInfo;
import com.jpsoft.smart.modules.base.entity.PersonDeviceRelation;
import com.jpsoft.smart.modules.base.entity.PersonInfo;
import com.jpsoft.smart.modules.base.service.CompanyInfoService;
import com.jpsoft.smart.modules.base.service.DeviceInfoService;
import com.jpsoft.smart.modules.base.service.PersonDeviceRelationService;
import com.jpsoft.smart.modules.base.service.PersonInfoService;
import com.jpsoft.smart.modules.common.utils.LApiUtil;
import com.jpsoft.smart.modules.common.utils.OSSUtil;
import com.jpsoft.smart.modules.constant.LApiConstant;
import com.jpsoft.smart.modules.lapi.dto.DataServersDTO;
import com.jpsoft.smart.modules.lapi.dto.ServerAddrDTO;
import com.jpsoft.smart.modules.lapi.request.LapiRequest;
import com.jpsoft.smart.modules.lapi.service.ILapiService;
import com.jpsoft.smart.modules.lapi.vo.*;
import io.netty.handler.codec.http.HttpMethod;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;

/**
 * @author 墨鱼_mo
 * @date 2020-3-15 9:37
 */
@Slf4j
@Service
public class LapiServiceImpl implements ILapiService {
    @Autowired
    private DeviceInfoService deviceInfoService;

    @Autowired
    private PersonInfoService personInfoService;

    @Autowired
    private PersonDeviceRelationService personDeviceRelationService;

    @Autowired
    private CompanyInfoService companyInfoService;

    @Autowired
    private LapiRequest lapiRequest;

    @Override
    public DeviceInfo findByIpAddressAndPort(String ipAddress, String port) throws Exception {
        JSONObject jsonObject = LApiUtil.getRequest(ipAddress + ":" + port + LApiConstant.DEVICEBASICINFO);

        JSONObject dataJson = jsonObject.getJSONObject("Response");
        JSONObject response = LApiUtil.getResponse(dataJson);
        DeviceInfo deviceInfo = new DeviceInfo();
        deviceInfo.setId(UUID.randomUUID().toString());
        deviceInfo.setIpAddress(ipAddress);
        deviceInfo.setPort(port);
        deviceInfo.setIsOnline(true);
        deviceInfo.setDeviceNo(response.getString("SerialNumber"));

        return deviceInfo;
    }

    @Override
    public Boolean keepAlive(String id) throws Exception {
        DeviceInfo deviceInfo = deviceInfoService.get(id);
        if (deviceInfo == null) {
            throw new Exception("设备未绑定");
        }
//        JSONObject jsonObject = LApiUtil.getRequest(deviceInfo.getIpAddress() + ":" + deviceInfo.getPort() + LApiConstant.KEEPALIVE);
        JSONObject jsonObject = lapiRequest.send(deviceInfo.getDeviceNo(), LApiConstant.KEEPALIVE, HttpMethod.GET, null);

        JSONObject dataJson = jsonObject.getJSONObject("Response");
        return true;
    }

    @Override
    public List<LapiResult> addPerson(Long id) throws Exception {

        PersonInfo personInfo = personInfoService.get(id);
        if (personInfo == null) {
            throw new Exception("人员不存在");
        }
        List<PersonDeviceRelation> listRelation = personDeviceRelationService.findByPersonId(id);
        if (listRelation.size() <= 0) {
            throw new Exception("人员未绑定设备");
        }
        CompanyInfo companyInfo = companyInfoService.get(personInfo.getCompanyId());
        List<DeviceInfo> deviceList = new ArrayList<>();
        for (PersonDeviceRelation personDeviceRelation : listRelation) {
            DeviceInfo deviceInfo = deviceInfoService.get(personDeviceRelation.getDeviceId());
            if (deviceInfo != null && !deviceInfo.getDelFlag()) {
                deviceList.add(deviceInfo);
            }
        }
        if (deviceList.size() <= 0) {
            throw new Exception("人员与设备绑定错误");
        }
        String base64Data = OSSUtil.downloadToBase64(personInfo.getFaceImageUrl());

        List<LapiResult> lapiResultList = new ArrayList<>();
        for (DeviceInfo deviceInfo : deviceList) {
            try {
                HashMap<String, Object> map = new HashMap<>();
                map.put("Num", 1L);
                HashMap<String, Object> map1 = new HashMap<>();
                map1.put("PersonID", personInfo.getId());
                map1.put("LastChange", new Date().getTime());
                map1.put("PersonCode", personInfo.getId().toString());
                map1.put("PersonName", personInfo.getName() + "_" + personInfo.getId().toString());
                map1.put("Remarks", companyInfo.getName());

                //TimeTemplateNum 首字符必须大写
                map1.put("TimeTemplateNum", 0L);

                map1.put("IdentificationNum", 1L);
                HashMap<String, Object> map2 = new HashMap<>();
                List listMap2 = new ArrayList();
                listMap2.add(map2);
                map2.put("Type", 0L);
                map2.put("Number", personInfo.getIdCard());
                map1.put("IdentificationList", listMap2);


                HashMap<String, Object> map3 = new HashMap<>();


                if (StringUtils.isNotBlank(base64Data)) {

                    Integer imageSize = base64Data.length();
                    //照片字节大小
                    Integer fileSize = imageSize - (imageSize / 8) * 2;
                    if (fileSize > 512 * 1024 || fileSize < 10 * 1024) {
                        throw new Exception("人员照片大小限定为10k~512k");
                    }
                    map1.put("ImageNum", 1);
                    map3.put("Size", imageSize);
                    map3.put("Data", base64Data);
                    map3.put("FaceID", personInfo.getId());
                    map3.put("Name", personInfo.getId() + ".jpg");
                } else {
                    throw new Exception("照片不存在");
                    /*map1.put("ImageNum", 0);
                    map3.put("FaceID", 0);
                    map3.put("Name", "");
                    map3.put("Size", 0);
                    map3.put("Data", "");*/
                }

                //照片base64长度

                List listMap3 = new ArrayList();
                listMap3.add(map3);
                map1.put("ImageList", listMap3);
                List listMap1 = new ArrayList();
                listMap1.add(map1);
                map.put("PersonInfoList", listMap1);


                //获取人员人脸库id
                String faceDbId = getFaceDbId(deviceInfo, companyInfo.getName());

//                JSONObject jsonObject = LApiUtil.postRequest(deviceInfo.getIpAddress() + ":" + deviceInfo.getPort() + LApiConstant.ADDPERSON + faceDbId + "/People", map);
                JSONObject jsonObject = lapiRequest.send(deviceInfo.getDeviceNo(), LApiConstant.ADDPERSON + faceDbId + "/People", HttpMethod.POST, map);

                JSONObject dataJson = jsonObject.getJSONObject("Response");
                JSONObject response = LApiUtil.getResponse(dataJson);
                if (dataJson.getInteger("ResponseCode") == 0 && dataJson.getInteger("StatusCode") == 0) {
                    Boolean success = isSuccessAddPerson(personInfo, deviceInfo, faceDbId);
                    HashMap<String, Object> dataMap = new HashMap<>();
                    dataMap.put("deviceId", deviceInfo.getId());
                    dataMap.put("deviceAliasName", deviceInfo.getAliasName());
                    dataMap.put("personName", personInfo.getName());
                    LapiResult lapiResult = new LapiResult();
                    if (success) {
                        lapiResult.setMsg("绑定成功");
                        lapiResult.setSuccess(true);
                        lapiResult.setData(dataMap);
                        lapiResultList.add(lapiResult);

                        try {
                            personDeviceRelationService.updateIsBound(personInfo.getId(), deviceInfo.getId(), true);
                        } catch (Exception e) {
                            log.error(e.getMessage(), e);
                        }
                    } else {
                        lapiResult.setMsg("绑定失败");
                        lapiResult.setSuccess(false);
                        lapiResult.setData(dataMap);
                        lapiResultList.add(lapiResult);
                        try {
                            personDeviceRelationService.updateIsBound(personInfo.getId(), deviceInfo.getId(), false);
                        } catch (Exception e) {
                            log.error(e.getMessage(), e);
                        }
                    }

                }
            } catch (Exception e) {
                log.error(e.getMessage());
                HashMap<String, Object> dataMap = new HashMap<>();
                dataMap.put("deviceId", deviceInfo.getId());
                dataMap.put("deviceAliasName", deviceInfo.getAliasName());
                dataMap.put("personName", personInfo.getName());
                LapiResult lapiResult = new LapiResult();
                lapiResult.setSuccess(false);
                lapiResult.setMsg(e.getMessage().contains("timed out") ? "设备连接超时" : e.getMessage());
                lapiResult.setData(dataMap);
                lapiResultList.add(lapiResult);
                personDeviceRelationService.updateIsBound(personInfo.getId(), deviceInfo.getId(), null);
            }
        }

        return lapiResultList;
    }

    @Override
    public List<LapiResult> addPersonForDeviceIds(Long id, String deviceIds) throws Exception {
        if (StringUtils.isBlank(deviceIds)) {
            throw new Exception("设备为空");
        }
        PersonInfo personInfo = personInfoService.get(id);
        CompanyInfo companyInfo = companyInfoService.get(personInfo.getCompanyId());
        String base64Data = OSSUtil.downloadToBase64(personInfo.getFaceImageUrl());

        List<String> deviceIdList = Arrays.asList(deviceIds.split(","));
        List<LapiResult> lapiResultList = new ArrayList<>();

        for (String deviceId : deviceIdList) {
            DeviceInfo deviceInfo = deviceInfoService.get(deviceId);
            try {
                HashMap<String, Object> map = new HashMap<>();
                map.put("Num", 1L);
                HashMap<String, Object> map1 = new HashMap<>();
                map1.put("PersonID", personInfo.getId());
                map1.put("LastChange", new Date().getTime());
                map1.put("PersonCode", personInfo.getId().toString());
                map1.put("PersonName", personInfo.getName() + "_" + personInfo.getId().toString());
                map1.put("Remarks", companyInfo.getName());

                //TimeTemplateNum 首字符必须大写
                map1.put("TimeTemplateNum", 0L);

                map1.put("IdentificationNum", 1L);
                HashMap<String, Object> map2 = new HashMap<>();
                List listMap2 = new ArrayList();
                listMap2.add(map2);
                map2.put("Type", 0L);
                map2.put("Number", personInfo.getIdCard());
                map1.put("IdentificationList", listMap2);


                HashMap<String, Object> map3 = new HashMap<>();


                if (StringUtils.isNotBlank(base64Data)) {
                    map1.put("ImageNum", 1L);
                    //照片base64长度
                    Integer imageSize = base64Data.length();
                    map3.put("Name", personInfo.getId() + ".jpg");
                    map3.put("Size", imageSize);
                    map3.put("Data", base64Data);
                    map3.put("FaceID", personInfo.getId());

                    //照片字节大小
                    Integer fileSize = imageSize - (imageSize / 8) * 2;

                    if (fileSize > 512 * 1024 || fileSize < 10 * 1024) {
                        throw new Exception("人员照片大小限定为10k~512k");
                    }
                } else {
                    throw new Exception("照片不存在");
                   /* map1.put("ImageNum", 0);
                    map3.put("FaceID", 0);
                    map3.put("Name", "");
                    map3.put("Size", 0);
                    map3.put("Data", "");*/
                }

                List listMap3 = new ArrayList();
                listMap3.add(map3);
                map1.put("ImageList", listMap3);
                List listMap1 = new ArrayList();
                listMap1.add(map1);
                map.put("PersonInfoList", listMap1);


                //获取人员人脸库id
                String faceDbId = getFaceDbId(deviceInfo, companyInfo.getName());

                //JSONObject jsonObject = LApiUtil.postRequest(deviceInfo.getIpAddress() + ":" + deviceInfo.getPort() + LApiConstant.ADDPERSON + faceDbId + "/People", map);
                JSONObject jsonObject = lapiRequest.send(deviceInfo.getDeviceNo(), LApiConstant.ADDPERSON + faceDbId + "/People", HttpMethod.POST, map);

                JSONObject dataJson = jsonObject.getJSONObject("Response");
                JSONObject response = LApiUtil.getResponse(dataJson);
                if (dataJson.getInteger("ResponseCode") == 0 && dataJson.getInteger("StatusCode") == 0) {
                    Boolean success = isSuccessAddPerson(personInfo, deviceInfo, faceDbId);
                    HashMap<String, Object> dataMap = new HashMap<>();
                    dataMap.put("deviceId", deviceInfo.getId());
                    dataMap.put("deviceAliasName", deviceInfo.getAliasName());
                    dataMap.put("personName", personInfo.getName());
                    LapiResult lapiResult = new LapiResult();
                    if (success) {
                        lapiResult.setMsg("绑定成功");
                        lapiResult.setSuccess(true);
                        lapiResult.setData(dataMap);
                        lapiResultList.add(lapiResult);
                        try {
                            personDeviceRelationService.updateIsBound(personInfo.getId(), deviceInfo.getId(), true);
                        } catch (Exception e) {
                            log.error(e.getMessage(), e);
                        }
                    } else {
                        lapiResult.setMsg("绑定失败");
                        lapiResult.setSuccess(false);
                        lapiResult.setData(dataMap);
                        lapiResultList.add(lapiResult);
                        try {
                            personDeviceRelationService.updateIsBound(personInfo.getId(), deviceInfo.getId(), false);
                        } catch (Exception e) {
                            log.error(e.getMessage(), e);
                        }
                    }


                }


            } catch (Exception e) {
                log.error(e.getMessage());
                HashMap<String, Object> dataMap = new HashMap<>();
                dataMap.put("deviceId", deviceInfo.getId());
                dataMap.put("deviceAliasName", deviceInfo.getAliasName());
                dataMap.put("personName", personInfo.getName());
                LapiResult lapiResult = new LapiResult();
                lapiResult.setSuccess(false);
                lapiResult.setMsg(e.getMessage().contains("timed out") ? "设备连接超时" : e.getMessage());
                lapiResult.setData(dataMap);
                lapiResultList.add(lapiResult);
                personDeviceRelationService.updateIsBound(personInfo.getId(), deviceInfo.getId(), null);
            }


        }


        return lapiResultList;
    }

    @Override
    public String getFaceDbId(DeviceInfo deviceInfo, String companyName) throws Exception {
        if (StringUtils.isBlank(companyName)) {
            throw new Exception("公司名称为空");
        }
        String faceDbId = "";

        //JSONObject jsonObject = LApiUtil.getRequest(deviceInfo.getIpAddress() + ":" + deviceInfo.getPort() + LApiConstant.GETFACEDB);
        JSONObject jsonObject = lapiRequest.send(deviceInfo.getDeviceNo(), LApiConstant.GETFACEDB, HttpMethod.GET, null);

        JSONObject dataJson = jsonObject.getJSONObject("Response");
        JSONObject response = LApiUtil.getResponse(dataJson);
        List listData = response.getJSONArray("LibList");
        if (listData.size() > 0) {
            for (int i = 0; i < listData.size(); i++) {
                JSONObject jsonObject1 = (JSONObject) JSONArray.toJSON(listData.get(i));
                String name = jsonObject1.getString("Name");
                if (StringUtils.isNotBlank(name) && companyName.equals(name)) {
                    faceDbId = jsonObject1.getString("ID");
                    break;
                }
            }
            //人员所属的公司库不存在，保存到默认库
            if (StringUtils.isBlank(faceDbId)) {
                for (int i = 0; i < listData.size(); i++) {
                    JSONObject jsonObject1 = (JSONObject) JSONArray.toJSON(listData.get(i));
                    String name = jsonObject1.getString("Name");
                    if ("默认员工库".equals(name)) {
                        faceDbId = jsonObject1.getString("ID");
                        break;
                    }
                }
                if (StringUtils.isBlank(faceDbId)) {
                    faceDbId = getDefaultLib(deviceInfo.getId());
                }


            }
        } else {
            faceDbId = getDefaultLib(deviceInfo.getId());
        }

        return faceDbId;
    }


    public Boolean isExistFaceDbId(DeviceInfo deviceInfo, String companyName) throws Exception {
        if (StringUtils.isBlank(companyName)) {
            throw new Exception("公司名称为空");
        }
        String faceDbId = "";
//        JSONObject jsonObject = LApiUtil.getRequest(deviceInfo.getIpAddress() + ":" + deviceInfo.getPort() + LApiConstant.GETFACEDB);
        JSONObject jsonObject = lapiRequest.send(deviceInfo.getDeviceNo(), LApiConstant.GETFACEDB, HttpMethod.GET, null);

        JSONObject dataJson = jsonObject.getJSONObject("Response");
        JSONObject response = LApiUtil.getResponse(dataJson);
        List listData = response.getJSONArray("LibList");
        if (listData.size() <= 0) {
            //
            return false;
        }
        for (int i = 0; i < listData.size(); i++) {
            JSONObject jsonObject1 = (JSONObject) JSONArray.toJSON(listData.get(i));
            String name = jsonObject1.getString("Name");
            if (StringUtils.isNotBlank(name) && companyName.equals(name)) {
                return true;
            }
        }
        return false;


    }

    @Override
    public List<LapiMsgResult> deletePerson(Long id) throws Exception {

        PersonInfo personInfo = personInfoService.getIgnoreDelFlag(id);
        if (personInfo == null) {
            throw new Exception("人员不存在");
        }
        List<PersonDeviceRelation> listRelation = personDeviceRelationService.findByPersonId(id);
        if (listRelation.size() <= 0) {
            throw new Exception("人员未绑定设备");
        }
        CompanyInfo companyInfo = companyInfoService.get(personInfo.getCompanyId());
        List<DeviceInfo> deviceList = new ArrayList<>();
        for (PersonDeviceRelation personDeviceRelation : listRelation) {
            DeviceInfo deviceInfo = deviceInfoService.get(personDeviceRelation.getDeviceId());
            if (deviceInfo != null && !deviceInfo.getDelFlag()) {
                deviceList.add(deviceInfo);
            }
        }
        if (deviceList.size() <= 0) {
            throw new Exception("人员与设备绑定错误");
        }

        List<LapiMsgResult> lapiMsgResults = new ArrayList<>();
        for (DeviceInfo deviceInfo : deviceList) {

            try {
                //获取人员人脸库id
                String faceDbId = getFaceDbId(deviceInfo, companyInfo.getName());
//                JSONObject jsonObject = LApiUtil.deleteRequest(deviceInfo.getIpAddress() + ":" + deviceInfo.getPort() + LApiConstant.DELETEPERSON + faceDbId + "/People/" + personInfo.getId() + "?LastChange=" + new Date().getTime());

                JSONObject jsonObject = lapiRequest.send(deviceInfo.getDeviceNo(),
                        LApiConstant.DELETEPERSON + faceDbId + "/People/" + personInfo.getId(), HttpMethod.DELETE, null);

                JSONObject dataJson = jsonObject.getJSONObject("Response");
                JSONObject response = LApiUtil.getResponse(dataJson);
                if (dataJson.getInteger("ResponseCode") == 0 && dataJson.getInteger("StatusCode") == 0) {
                    LapiMsgResult lapiMsgResult = new LapiMsgResult();
                    lapiMsgResult.setPersonName(personInfo.getName());
                    lapiMsgResult.setSuccess(true);
                    lapiMsgResult.setAliasName(deviceInfo.getAliasName());
                    lapiMsgResults.add(lapiMsgResult);
                }

            } catch (Exception e) {

                log.error(e.getMessage(), e);
                LapiMsgResult lapiMsgResult = new LapiMsgResult();
                lapiMsgResult.setPersonName(personInfo.getName());
                lapiMsgResult.setSuccess(false);
                lapiMsgResult.setMsg(e.getMessage());
                lapiMsgResult.setAliasName(deviceInfo.getAliasName());
                lapiMsgResults.add(lapiMsgResult);

            }


        }

        return lapiMsgResults;
    }

    @Override
    public List<LapiMsgResult> deleteDevicePerson(Long id, String deviceIds) throws Exception {
        if (StringUtils.isBlank(deviceIds)) {
            throw new Exception("相关设备不能为空");
        }
        PersonInfo personInfo = personInfoService.getIgnoreDelFlag(id);
        if (personInfo == null) {
            throw new Exception("人员信息不存在");
        }
        CompanyInfo companyInfo = companyInfoService.get(personInfo.getCompanyId());
        if (companyInfo == null) {
            throw new Exception("人员没有相应公司");
        }
        List<LapiMsgResult> lapiMsgResults = new ArrayList<>();
        List<String> deviceIdList = Arrays.asList(deviceIds.split(","));
        for (String deviceId : deviceIdList) {
            DeviceInfo deviceInfo = deviceInfoService.get(deviceId);
            try {
                String faceDbId = getFaceDbId(deviceInfo, companyInfo.getName());
//                JSONObject jsonObject = LApiUtil.deleteRequest(deviceInfo.getIpAddress() + ":" + deviceInfo.getPort() + LApiConstant.DELETEPERSON + faceDbId + "/People/" + personInfo.getId() + "?LastChange=" + new Date().getTime());

                JSONObject jsonObject = lapiRequest.send(deviceInfo.getDeviceNo(),
                        LApiConstant.DELETEPERSON + faceDbId + "/People/" + personInfo.getId(), HttpMethod.DELETE, null);

                JSONObject dataJson = jsonObject.getJSONObject("Response");
                JSONObject response = LApiUtil.getResponse(dataJson);
                if (dataJson.getInteger("ResponseCode") == 0 && dataJson.getInteger("StatusCode") == 0) {
                    LapiMsgResult lapiMsgResult = new LapiMsgResult();
                    lapiMsgResult.setPersonName(personInfo.getName());
                    lapiMsgResult.setSuccess(true);
                    lapiMsgResult.setAliasName(deviceInfo.getAliasName());
                    lapiMsgResults.add(lapiMsgResult);
                }
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                LapiMsgResult lapiMsgResult = new LapiMsgResult();
                lapiMsgResult.setPersonName(personInfo.getName());
                lapiMsgResult.setSuccess(false);
                lapiMsgResult.setMsg(e.getMessage());
                lapiMsgResult.setAliasName(deviceInfo.getAliasName());
                lapiMsgResults.add(lapiMsgResult);

            }


        }

        return lapiMsgResults;
    }

    @Override
    public Boolean phoneOpenDoor(String deviceInfoId) throws Exception {
        DeviceInfo deviceInfo = deviceInfoService.get(deviceInfoId);
        if (deviceInfo == null) {
            throw new Exception("设备不存在");
        }
        if (deviceInfo.getDelFlag()) {
            throw new Exception("设备已删除");
        }

//      JSONObject jsonObject = LApiUtil.putRequest(deviceInfo.getIpAddress() + ":" + deviceInfo.getPort() + LApiConstant.REMOTEOPENED);
        JSONObject jsonObject = lapiRequest.send(deviceInfo.getDeviceNo(), LApiConstant.REMOTEOPENED, HttpMethod.PUT, null);
        JSONObject dataJson = jsonObject.getJSONObject("Response");
        JSONObject response = LApiUtil.getResponse(dataJson);
        return true;
    }

    @Override
    public Boolean reBoot(String deviceInfoId) throws Exception {
        DeviceInfo deviceInfo = deviceInfoService.get(deviceInfoId);
        if (deviceInfo == null) {
            throw new Exception("设备不存在");
        }
        if (deviceInfo.getDelFlag()) {
            throw new Exception("设备已删除");
        }
//        JSONObject jsonObject = LApiUtil.putRequest(deviceInfo.getIpAddress() + ":" + deviceInfo.getPort() + LApiConstant.REBOOT);
        JSONObject jsonObject = lapiRequest.send(deviceInfo.getDeviceNo(), LApiConstant.REBOOT, HttpMethod.PUT, null);
        JSONObject dataJson = jsonObject.getJSONObject("Response");
        JSONObject response = LApiUtil.getResponse(dataJson);
        return true;
    }

    @Override
    public List<LapiResult> addCompanyLib(String companyInfoId, String deviceInfoIds) throws Exception {
        CompanyInfo companyInfo = companyInfoService.get(companyInfoId);
        if (companyInfo == null) {
            throw new Exception("公司不存在");
        }
        if (StringUtils.isBlank(deviceInfoIds)) {
            throw new Exception("请选择设备");
        }
        List<String> idList = Arrays.asList(deviceInfoIds.split(","));
        List<LapiResult> lapiResultList = new ArrayList<>();
        for (String deviceInfoId : idList) {
            DeviceInfo deviceInfo = deviceInfoService.get(deviceInfoId);
            LapiResult lapiResult = new LapiResult();
            try {
                if (deviceInfo == null) {
                    throw new Exception("设备不存在");
                }
                Boolean isExist = isExistFaceDbId(deviceInfo, companyInfo.getName());
                if (isExist) {
                    throw new Exception("已有人脸库不可重复创建");
                }
                HashMap map = new HashMap();
                map.put("BelongIndex", "");
                map.put("LastChange", new Date().getTime());
                map.put("MemberNum", 0);
                map.put("Name", companyInfo.getName());
                map.put("Type", 3);
                //JSONObject jsonObject = LApiUtil.postRequest(deviceInfo.getIpAddress() + ":" + deviceInfo.getPort() + LApiConstant.ADDLIBRARY, map);
                JSONObject jsonObject = lapiRequest.send(deviceInfo.getDeviceNo(), LApiConstant.ADDLIBRARY, HttpMethod.POST, map);

                JSONObject dataJson = jsonObject.getJSONObject("Response");
                JSONObject response = LApiUtil.getResponse(dataJson);
                lapiResult.setMsg("添加成功");
                lapiResult.setSuccess(true);
                HashMap dataMap = new HashMap<>();
                dataMap.put("CompanyId", companyInfoId);
                dataMap.put("CompanyName", companyInfo.getName());
                dataMap.put("deviceId", deviceInfoId);
                dataMap.put("LibraryId", response.getString("ID"));
                lapiResult.setData(dataMap);

            } catch (Exception e) {
                log.error(e.getMessage());
                lapiResult.setMsg(e.getMessage());
                lapiResult.setSuccess(false);
                lapiResult.setData("");
            }

            lapiResultList.add(lapiResult);


        }


        return lapiResultList;
    }

    public String getDefaultLib(String deviceInfoId) throws Exception {

        if (StringUtils.isBlank(deviceInfoId)) {
            throw new Exception("请选择设备");
        }
        DeviceInfo deviceInfo = deviceInfoService.get(deviceInfoId);
        try {
            if (deviceInfo == null) {
                throw new Exception("设备不存在");
            }


            HashMap map = new HashMap();
            map.put("BelongIndex", "");
            map.put("LastChange", new Date().getTime());
            map.put("MemberNum", 0);
            map.put("Name", "默认员工库");
            map.put("Type", 3);
//            JSONObject jsonObject = LApiUtil.postRequest(deviceInfo.getIpAddress() + ":" + deviceInfo.getPort() + LApiConstant.ADDLIBRARY, map);
            JSONObject jsonObject = lapiRequest.send(deviceInfo.getDeviceNo(), LApiConstant.ADDLIBRARY, HttpMethod.POST, map);
            JSONObject dataJson = jsonObject.getJSONObject("Response");
            JSONObject response = LApiUtil.getResponse(dataJson);
            return response.getString("ID");

        } catch (Exception e) {
            log.error(e.getMessage());

        }
        return "";
    }

    @Override
    public LapiResult deleteCompanyLib(String deviceInfoId, String libraryId) throws Exception {
        if (StringUtils.isBlank(deviceInfoId)) {
            throw new Exception("设备id为空");
        }
        if (StringUtils.isBlank(libraryId)) {
            throw new Exception("人脸库id为空");
        }
        DeviceInfo deviceInfo = deviceInfoService.get(deviceInfoId);
        if (deviceInfo == null) {
            throw new Exception("设备不存在");
        }

        LapiResult lapiResult = new LapiResult();
        try {
//          JSONObject jsonObject = LApiUtil.deleteRequest(deviceInfo.getIpAddress() + ":" + deviceInfo.getPort() + LApiConstant.DELETELIBRARY + libraryId);
            JSONObject jsonObject = lapiRequest.send(deviceInfo.getDeviceNo(),
                    LApiConstant.DELETELIBRARY + libraryId, HttpMethod.DELETE, null);

            JSONObject dataJson = jsonObject.getJSONObject("Response");
            JSONObject response = LApiUtil.getResponse(dataJson);
            lapiResult.setSuccess(true);
            lapiResult.setMsg("删除成功");
            lapiResult.setData("");
        } catch (Exception e) {
            lapiResult.setMsg(e.getMessage());
            lapiResult.setSuccess(false);
            lapiResult.setData("");
        }


        return lapiResult;
    }

    @Override
    public LapiResult updateCompanyLib(String companyId, String deviceInfoId, String libraryId) throws Exception {
        if (StringUtils.isBlank(companyId)) {
            throw new Exception("公司id为空");
        }
        CompanyInfo companyInfo = companyInfoService.get(companyId);
        if (companyInfo == null) {
            throw new Exception("公司不存在");
        }
        if (StringUtils.isBlank(deviceInfoId)) {
            throw new Exception("设备id为空");
        }
        if (StringUtils.isBlank(libraryId)) {
            throw new Exception("人脸库id为空");
        }
        DeviceInfo deviceInfo = deviceInfoService.get(deviceInfoId);
        if (deviceInfo == null) {
            throw new Exception("设备不存在");
        }
        HashMap map = new HashMap();
        map.put("Num", 1);
        List list = new ArrayList();
        HashMap map1 = new HashMap();
        map1.put("ID", libraryId);
        map1.put("LastChange", new Date().getTime());
        map1.put("Name", companyInfo.getName());
        map1.put("Type", 3);
        map1.put("BelongIndex", "");
        list.add(map1);
        map.put("LibList", list);
//        JSONObject jsonObject = LApiUtil.putRequest(deviceInfo.getIpAddress() + ":" + deviceInfo.getPort() + LApiConstant.UPDATALIBRARY, map);
        JSONObject jsonObject = lapiRequest.send(deviceInfo.getDeviceNo(), LApiConstant.UPDATALIBRARY, HttpMethod.PUT, map);
        JSONObject dataJson = jsonObject.getJSONObject("Response");
        JSONObject response = LApiUtil.getResponse(dataJson);

        HashMap dataMap = new HashMap<>();
        dataMap.put("deviceId", deviceInfo.getId());
        dataMap.put("deviceName", deviceInfo.getAliasName());
        LapiResult lapiResult = new LapiResult();
        lapiResult.setSuccess(true);
        lapiResult.setMsg("更新成功");
        lapiResult.setData(dataMap);
        return lapiResult;
    }

    @Override
    public List<LapiResult> deleteDeviceAllCompanyLib(String deviceInfoId) throws Exception {
        if (StringUtils.isBlank(deviceInfoId)) {
            throw new Exception("设备id为空");
        }
        DeviceInfo deviceInfo = deviceInfoService.get(deviceInfoId);
        if (deviceInfo == null) {
            throw new Exception("设备不存在");
        }

        //JSONObject jsonObject = LApiUtil.getRequest(deviceInfo.getIpAddress() + ":" + deviceInfo.getPort() + LApiConstant.UPDATALIBRARY);
        JSONObject jsonObject = lapiRequest.send(deviceInfo.getDeviceNo(), LApiConstant.UPDATALIBRARY, HttpMethod.GET, null);

        JSONObject dataJson = jsonObject.getJSONObject("Response");
        JSONObject response = LApiUtil.getResponse(dataJson);
        List<LibraryVo> list = JSONObject.parseArray(response.getString("LibList"), LibraryVo.class);
        if (list.size() <= 0) {
            throw new Exception("设备没有配置人员库");
        }
        List<LapiResult> lapiResultList = new ArrayList<>();
        for (LibraryVo libraryVo : list) {
            LapiResult lapiResult = new LapiResult();
            try {
//                JSONObject jsonObject1 = LApiUtil.deleteRequest(deviceInfo.getIpAddress() + ":" + deviceInfo.getPort() + LApiConstant.DELETELIBRARY + libraryVo.getID());
                JSONObject jsonObject1 = lapiRequest.send(deviceInfo.getDeviceNo(),
                        LApiConstant.DELETELIBRARY + libraryVo.getID(), HttpMethod.DELETE, null);

                JSONObject dataJson1 = jsonObject1.getJSONObject("Response");
                JSONObject response1 = LApiUtil.getResponse(dataJson1);
                lapiResult.setSuccess(true);
                lapiResult.setMsg("删除成功");
                lapiResult.setData("");
            } catch (Exception e) {
                lapiResult.setMsg(e.getMessage());
                lapiResult.setSuccess(false);
                lapiResult.setData("");
            }
            lapiResultList.add(lapiResult);
        }


        return lapiResultList;
    }

    @Override
    public Boolean isSuccessAddPerson(PersonInfo personInfo, DeviceInfo deviceInfo, String faceDbId) {
        HashMap map = new HashMap();
        map.put("Num", 1);
        map.put("Limit", 3);
        map.put("Offset", 0);
        List list = new ArrayList();
        HashMap map1 = new HashMap();
        map1.put("QryType", 27);
        map1.put("QryCondition", 0);
        map1.put("QryData", personInfo.getId().toString());
        list.add(map1);
        map.put("QueryInfos", list);
        try {
            //JSONObject jsonObject = LApiUtil.postRequest(deviceInfo.getIpAddress() + ":" + deviceInfo.getPort() + LApiConstant.ADDPERSON + faceDbId + "/People/Info", map);
            JSONObject jsonObject = lapiRequest.send(deviceInfo.getDeviceNo(), LApiConstant.ADDPERSON + faceDbId + "/People/Info", HttpMethod.POST, map);

            JSONObject dataJson = jsonObject.getJSONObject("Response");
            JSONObject response = LApiUtil.getResponse(dataJson);
            Integer num = response.getInteger("Total");
            if (num > 0) {

                try {
                    personDeviceRelationService.updateIsBound(personInfo.getId(), deviceInfo.getId(), true);
                } catch (Exception e) {
                    log.error(e.getMessage(), e);
                }

                return true;
            } else {
                try {
                    personDeviceRelationService.updateIsBound(personInfo.getId(), deviceInfo.getId(), false);
                } catch (Exception e) {
                    log.error(e.getMessage(), e);
                }
            }
        } catch (Exception e) {

            personDeviceRelationService.updateIsBound(personInfo.getId(), deviceInfo.getId(), null);
            return false;
        }


        return false;
    }

    @Override
    public Boolean isPersonImageExist(PersonInfo personInfo, DeviceInfo deviceInfo, String faceDbId) {
        HashMap map = new HashMap();
        map.put("Num", 1);
        map.put("Limit", 3);
        map.put("Offset", 0);
        List list = new ArrayList();
        HashMap map1 = new HashMap();
        map1.put("QryType", 27);
        map1.put("QryCondition", 0);
        map1.put("QryData", personInfo.getId().toString());
        list.add(map1);
        map.put("QueryInfos", list);
        try {

            JSONObject jsonObject = lapiRequest.send(deviceInfo.getDeviceNo(), LApiConstant.ADDPERSON + faceDbId + "/People/Info", HttpMethod.POST, map);

            JSONObject dataJson = jsonObject.getJSONObject("Response");
            JSONObject response = LApiUtil.getResponse(dataJson);
            Integer num = response.getInteger("Total");
            if (num > 0) {
                JSONObject personList = response.getJSONObject("PersonList");

                JSONArray personInfoList = personList.getJSONArray("PersonInfoList");
                Map map2 = (Map) personInfoList.get(0);

                Integer imageNum = (Integer) map2.get("ImageNum");
                if (imageNum > 0) {
                    return true;
                }

            }
        } catch (Exception e) {

            return false;
        }


        return false;
    }

    @Override
    public Map<String, Object> getDeviceRule(String deviceInfoId) throws Exception {

        DeviceInfo deviceInfo = deviceInfoService.get(deviceInfoId);
        if (deviceInfo == null) {
            throw new Exception("设备不存在");
        }
        if (!deviceInfo.getIsOnline()) {
            throw new Exception("设备离线，请联系管理员");
        }
        //JSONObject jsonObject = LApiUtil.getRequest(deviceInfo.getIpAddress() + ":" + deviceInfo.getPort() + LApiConstant.RULE);
        JSONObject jsonObject = lapiRequest.send(deviceInfo.getDeviceNo(), LApiConstant.RULE, HttpMethod.GET, null);

        JSONObject dataJson = jsonObject.getJSONObject("Response");
        JSONObject response = LApiUtil.getResponse(dataJson);
        return response;
    }

    @Override
    public Boolean setDeviceRule(String deviceInfoId, Integer needCap, Integer needMask, Integer measureTemperature, Integer type, BigDecimal minTemper, BigDecimal maxTemper, BigDecimal alarmThreshold) throws Exception {
        DeviceInfo deviceInfo = deviceInfoService.get(deviceInfoId);
        if (deviceInfo == null) {
            throw new Exception("设备不存在");
        }
        if (!deviceInfo.getIsOnline()) {
            throw new Exception("设备离线，请联系管理员");
        }
        HashMap<String, Object> map = new HashMap<>();
        map.put("Num", 3);
        List list = new ArrayList();
        //安全帽数据
        HashMap<String, Object> map1 = new HashMap<>();
        map1.put("DoorOpenMode", 0);
        map1.put("Mode", 1);
        map1.put("Enabled", needCap);
        //口罩数据
        HashMap<String, Object> map2 = new HashMap<>();
        map2.put("DoorOpenMode", 0);
        map2.put("Mode", 2);
        map2.put("Enabled", needMask);
        //测温数据
        HashMap<String, Object> map3 = new HashMap<>();
        map3.put("DoorOpenMode", 0);
        map3.put("Mode", 3);
        map3.put("Enabled", measureTemperature);
        list.add(map1);
        list.add(map2);
        list.add(map3);
        map.put("RuleList", list);
        if (measureTemperature == 1) {
            HashMap<String, Object> temperatureMap = new HashMap<>();
            temperatureMap.put("Type", type);
            temperatureMap.put("Minimum", minTemper);
            temperatureMap.put("Maximum", maxTemper);
            temperatureMap.put("AlarmThreshold", alarmThreshold);
            map.put("TemperatureRule", temperatureMap);
        }

//      JSONObject jsonObject = LApiUtil.putRequest(deviceInfo.getIpAddress() + ":" + deviceInfo.getPort() + LApiConstant.RULE, map);
        JSONObject jsonObject = lapiRequest.send(deviceInfo.getDeviceNo(), LApiConstant.RULE, HttpMethod.PUT, map);

        JSONObject dataJson = jsonObject.getJSONObject("Response");
        JSONObject response = LApiUtil.getResponse(dataJson);

        return true;
    }

    @Override
    public Boolean updateDeviceNo(String oldDeviceNo, String newDeviceNo) throws Exception{
        DeviceInfo deviceInfo = deviceInfoService.getByDeviceNo(oldDeviceNo);
        DataServersDTO dataServersDTO = getDeviceDateServer(oldDeviceNo);
        if (dataServersDTO !=null){
            dataServersDTO.setTollgateID(newDeviceNo);
            List<DataServersDTO> dataServersDTOList = new ArrayList<>();
            dataServersDTOList.add(dataServersDTO);
            HashMap<String,Object> map = new HashMap<>();
            map.put("Num",1);
            map.put("DataServerList",dataServersDTOList);
            JSONObject jsonObject = lapiRequest.send(oldDeviceNo, LApiConstant.DATASERVERS, HttpMethod.PUT, map);
            JSONObject dataJson = jsonObject.getJSONObject("Response");
            JSONObject response = LApiUtil.getResponse(dataJson);

            if (dataJson.getString("ResponseString").equals("Succeed") && dataJson.getString("StatusString").equals("Succeed")){
                deviceInfo.setDeviceNo(newDeviceNo);
                deviceInfo.setUpdateTime(new Date());
                deviceInfoService.update(deviceInfo);
            }

            return true;
        }
        return false;
    }

    @Override
    public DataServersDTO getDeviceDateServer(String deviceNo) throws Exception {
        DeviceInfo deviceInfo = deviceInfoService.getByDeviceNo(deviceNo);
        if (deviceInfo == null) {
            throw new Exception("设备不存在");
        }
        if (!deviceInfo.getIsOnline()) {
            throw new Exception("设备离线");
        }
        JSONObject jsonObject = lapiRequest.send(deviceNo, LApiConstant.DATASERVERS, HttpMethod.GET, null);
        JSONObject dataJson = jsonObject.getJSONObject("Response");
        JSONObject response = LApiUtil.getResponse(dataJson);
        String s = response.getString("DataServerList");
        List<DataServersDTO> dataServersDTOList = JSONObject.parseArray(s,DataServersDTO.class);
        if (dataServersDTOList.size() > 0) {
            for (DataServersDTO dataServersDTO : dataServersDTOList) {
                if (dataServersDTO.getEnabled() > 0) {
                    return dataServersDTO;
                }
            }
        }
        return null;
    }


}
