package com.jpsoft.smart.lapi;

import com.jpsoft.smart.lapi.handler.*;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.codec.http.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import javax.annotation.PreDestroy;

@Slf4j
@Component
public class LapiTcpServer {
    @Autowired
    private PersonVerificationHandler personVerificationHandler;

    @Autowired
    private HeartReportHandler heartReportHandler;

    @Autowired
    private ResponseDataHandler responseDataHandler;

    @Value("${server.port}")
    private Integer port;

    private Channel serverChannel;

    public void start(int port){
        NioEventLoopGroup bossGroup = new NioEventLoopGroup();
        NioEventLoopGroup workerGroup = new NioEventLoopGroup();

        ServerBootstrap b = new ServerBootstrap();

        b.group(bossGroup,workerGroup)
                .channel(NioServerSocketChannel.class)
                .childHandler(new ChannelInitializer(){
                    @Override
                    protected void initChannel(Channel channel) throws Exception {
//                        channel.pipeline().addLast(new HttpRequestDecoder());
//                        channel.pipeline().addLast(new HttpResponseDecoder());
//                        channel.pipeline().addLast(new ParseResponseHandler());
//                        channel.pipeline().addLast(new HttpClientCodec());
//                        channel.pipeline().addLast(new HttpServerCodec());
                        channel.pipeline().addLast(new HttpDecoder());
                        channel.pipeline().addLast("aggregator", new HttpObjectAggregator(10 * 1024 * 1024));
                        channel.pipeline().addLast(new ParseHttpHandler());
                        channel.pipeline().addLast(heartReportHandler);
                        channel.pipeline().addLast(personVerificationHandler);
                        channel.pipeline().addLast(responseDataHandler);
                        channel.pipeline().addLast(new HttpRequestEncoder());
                        channel.pipeline().addLast(new HttpResponseEncoder());
                    }
                });

        ChannelFuture channelFuture = b.bind("0.0.0.0",port);
        serverChannel = channelFuture.channel();

        channelFuture.addListener((future)-> {
            if(future.isSuccess()){
                log.warn(String.format("netty 已监听%d端口",port));
            }
        });
    }

    @PreDestroy
    public void stop(){
        if (serverChannel!=null && serverChannel.isOpen()){
            serverChannel.close();
        }
    }
}
