package com.jpsoft.smart.lapi.handler;

import com.jpsoft.smart.modules.lapi.channel.LapiAttrKeys;
import com.jpsoft.smart.modules.lapi.channel.LapiChannel;
import com.jpsoft.smart.lapi.dto.HeartReportInfo;
import com.jpsoft.smart.modules.base.service.DeviceInfoService;
import com.jpsoft.smart.modules.sys.entity.SysLog;
import com.jpsoft.smart.modules.sys.service.SysLogService;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import lombok.extern.slf4j.Slf4j;
import org.joda.time.DateTime;
import org.joda.time.Minutes;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.net.SocketAddress;
import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Slf4j
@Component
@ChannelHandler.Sharable
public class HeartReportHandler extends SimpleChannelInboundHandler<HeartReportInfo> {
    @Autowired
    private DeviceInfoService deviceInfoService;

    @Autowired
    private Map<String, LapiChannel> activeLapiChannelMap;

    @Autowired
    private SysLogService sysLogService;

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, HeartReportInfo heartReportInfo) throws Exception {
        log.warn(heartReportInfo.toString());
        String ipAndPort = String.valueOf(ctx.channel().remoteAddress());
        String deviceNo = heartReportInfo.getDeviceCode();
        String ip = ipAndPort.substring(ipAndPort.indexOf("/")+1,ipAndPort.indexOf(":"));

        log.warn("收到设备心跳，deviceNo=" + deviceNo);
        log.warn("ip =="+ip);

        DateTime lastHeartTime = ctx.channel().attr(LapiAttrKeys.LAST_HEART_TIME).get();

        boolean needWrite = false;

        if(lastHeartTime==null){
            ctx.channel().attr(LapiAttrKeys.LAST_HEART_TIME).set(DateTime.now());
            needWrite = true;
        }
        else{
            int minutes = Minutes.minutesBetween(lastHeartTime,DateTime.now()).getMinutes();

            if(minutes>=10){
                ctx.channel().attr(LapiAttrKeys.LAST_HEART_TIME).set(DateTime.now());
                needWrite = true;
            }
        }

        if(needWrite) {
            new Thread(()->{
                SysLog sysLog = new SysLog();
                sysLog.setPointcut(deviceNo);
                sysLog.setUrl("/LAPI/V1.0/PACS/Controller/HeartReportInfo");
                sysLog.setData(heartReportInfo.toString());
                sysLog.setRemark("收到设备心跳");
                SocketAddress remoteAddress = ctx.channel().remoteAddress();
                sysLog.setRemoteIp(remoteAddress.toString());
                sysLog.setCreateTime(new Date());
                sysLogService.insert(sysLog);
            }).start();
        }

        deviceInfoService.updateByDeviceNo(deviceNo,ip);

        //新连接，则设置连接对应设备号
        if(ctx.channel().attr(LapiAttrKeys.DEVICE_NO).get()==null){
            ctx.channel().attr(LapiAttrKeys.DEVICE_NO).set(deviceNo);
        }

        if (activeLapiChannelMap.containsKey(deviceNo)) {
            activeLapiChannelMap.get(deviceNo).setChannel(ctx.channel());
        }
        else {
            LapiChannel lapiChannel = new LapiChannel();
            lapiChannel.setChannel(ctx.channel());
            lapiChannel.setReceivePacketQueueMap(new ConcurrentHashMap<>());

            activeLapiChannelMap.put(deviceNo, lapiChannel);
        }
    }
}
