package com.jpsoft.smart.listener;

import com.jpsoft.smart.modules.base.entity.PersonInfo;
import com.jpsoft.smart.modules.base.service.PersonInfoService;
import com.jpsoft.smart.modules.lapi.service.ILapiService;
import com.jpsoft.smart.modules.lapi.vo.LapiMsgResult;
import com.jpsoft.smart.modules.lapi.vo.LapiResult;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * concurrency 表示并发数
 */
@Slf4j
@Component
@RabbitListener(queues = "importImageQueue",concurrency = "2")
public class ImportImageListener {
    @Autowired
    private ILapiService lapiService;

    @Autowired
    private PersonInfoService personInfoService;

    @RabbitHandler
    public void process(Long personId) {
        System.out.println("收到用户编号:" + personId);

        try {
            List<LapiResult> msgResultList = lapiService.addPerson(personId);

            StringBuilder sb = new StringBuilder();

            boolean success = true;

            for (int i = 0; i < msgResultList.size(); i++) {
                LapiResult lapiMsgResult = msgResultList.get(i);
                Map lapiMsgMap = (Map)lapiMsgResult.getData();
                if (lapiMsgResult.isSuccess()) {
                    sb.append(lapiMsgMap.get("deviceAliasName") + "绑定人脸信息成功!");
                } else {
                    sb.append(lapiMsgMap.get("deviceAliasName") + "绑定人脸信息失败!" + lapiMsgResult.getMsg());
                }

                if (i != msgResultList.size() - 1) {
                    sb.append(",");
                } else {
                    sb.append("。");
                }

                success &= lapiMsgResult.isSuccess();
            }

            PersonInfo personInfo = personInfoService.get(personId);

            personInfo.setFaceBound(success);
            personInfo.setUpdateTime(new Date());

            personInfoService.update(personInfo);

            log.warn(sb.toString());
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);
        }
    }
}
