package com.jpsoft.smart.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.smart.modules.base.entity.AlarmConfig;
import com.jpsoft.smart.modules.base.entity.CompanyInfo;
import com.jpsoft.smart.modules.base.entity.OwnerInfo;
import com.jpsoft.smart.modules.base.entity.PersonInfo;
import com.jpsoft.smart.modules.base.service.AlarmConfigService;
import com.jpsoft.smart.modules.base.service.AlarmConfigService;
import com.jpsoft.smart.modules.base.service.CompanyInfoService;
import com.jpsoft.smart.modules.base.service.OwnerInfoService;
import com.jpsoft.smart.modules.business.entity.WorkAttendance;
import com.jpsoft.smart.modules.common.dto.MessageResult;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.modules.common.utils.PojoUtils;
import com.jpsoft.smart.modules.common.utils.WechatMessageUtil;
import com.jpsoft.smart.modules.sys.entity.User;
import com.jpsoft.smart.modules.sys.service.UserService;
import com.jpsoft.smart.modules.wechat.entity.AccessControl;
import com.jpsoft.smart.modules.wechat.service.IAccessControlService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.joda.time.Days;
import org.joda.time.Months;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.web.bind.annotation.*;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/base/alarmConfig")
public class AlarmConfigController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private AlarmConfigService alarmConfigService;

    @Autowired
    private UserService userService;

    @Autowired
    private CompanyInfoService companyInfoService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<AlarmConfig> create(){
        MessageResult<AlarmConfig> msgResult = new MessageResult<>();

        AlarmConfig alarmConfig = new AlarmConfig();

        msgResult.setData(alarmConfig);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<AlarmConfig> add(@RequestBody AlarmConfig alarmConfig,@RequestAttribute String subject){
        MessageResult<AlarmConfig> msgResult = new MessageResult<>();

        try {
            alarmConfig.setId(UUID.randomUUID().toString());
            alarmConfig.setDelFlag(false);
            alarmConfig.setCreateBy(subject);
            alarmConfig.setCreateTime(new Date());

            if(StringUtils.isNotEmpty(alarmConfig.getWeekdays())){
                String[] weekdays =  alarmConfig.getWeekdays().split(",");
                String sortWeekDays = Arrays.stream(weekdays).sorted(Comparator.naturalOrder()).collect(Collectors.joining(","));
                alarmConfig.setWeekdays(sortWeekDays);
            }

            int affectCount = alarmConfigService.insert(alarmConfig);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(alarmConfig);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<AlarmConfig> edit(@PathVariable("id") String id){
        MessageResult<AlarmConfig> msgResult = new MessageResult<>();

        try {
            AlarmConfig alarmConfig = alarmConfigService.get(id);

            if (alarmConfig != null) {
                msgResult.setResult(true);
                msgResult.setData(alarmConfig);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<AlarmConfig> update(@RequestBody AlarmConfig alarmConfig,@RequestAttribute String subject){
        MessageResult<AlarmConfig> msgResult = new MessageResult<>();

        try {
            alarmConfig.setUpdateBy(subject);
            alarmConfig.setUpdateTime(new Date());

            if(StringUtils.isNotEmpty(alarmConfig.getWeekdays())){
                String[] weekdays =  alarmConfig.getWeekdays().split(",");
                String sortWeekDays = Arrays.stream(weekdays).sorted(Comparator.naturalOrder()).collect(Collectors.joining(","));
                alarmConfig.setWeekdays(sortWeekDays);
            }

            int affectCount = alarmConfigService.update(alarmConfig);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(alarmConfig);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            AlarmConfig alarmConfig = alarmConfigService.get(id);
            alarmConfig.setDelFlag(true);
            alarmConfig.setUpdateBy(subject);
            alarmConfig.setUpdateTime(new Date());

            int affectCount = alarmConfigService.update(alarmConfig);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                AlarmConfig alarmConfig = alarmConfigService.get(id);
                alarmConfig.setDelFlag(true);
                alarmConfig.setUpdateBy(subject);
                alarmConfig.setUpdateTime(new Date());

                affectCount += alarmConfigService.update(alarmConfig);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="companyId",defaultValue="") String companyId,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        User user = userService.get(subject);
        CompanyInfo companyInfo = companyInfoService.get(user.getCompanyId());

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("a.company_id","asc"));
        sortList.add(new Sort("a.weekdays_","asc"));
        sortList.add(new Sort("a.start_time","asc"));

        if (StringUtils.isNotEmpty(companyId)) {
            searchParams.put("companyId",companyId);
        }
        else{
            if (!userService.hasRole(subject,"SYSADMIN")) {
                searchParams.put("companyId",companyInfo.getId());
            }
        }

        Page<AlarmConfig> page = alarmConfigService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
