package com.jpsoft.smart.modules.business.controller;

import com.github.pagehelper.Page;
import com.jpsoft.smart.config.OSSConfig;
import com.jpsoft.smart.modules.base.entity.AlarmConfig;
import com.jpsoft.smart.modules.base.entity.CompanyInfo;
import com.jpsoft.smart.modules.base.entity.PersonInfo;
import com.jpsoft.smart.modules.base.service.AlarmConfigService;
import com.jpsoft.smart.modules.base.service.CompanyInfoService;
import com.jpsoft.smart.modules.base.service.PersonInfoService;
import com.jpsoft.smart.modules.common.utils.OSSUtil;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.modules.common.dto.MessageResult;
import com.jpsoft.smart.modules.business.entity.WorkAttendance;
import com.jpsoft.smart.modules.business.service.WorkAttendanceService;
import com.jpsoft.smart.schduled.UnmeasureTemperatureAlarmTask;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFRichTextString;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFRichTextString;
import org.joda.time.DateTime;
import org.joda.time.Days;
import org.joda.time.Months;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.data.redis.core.ListOperations;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.web.bind.annotation.*;
import java.io.*;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/business/workAttendance")
@Api(description = "考勤统计表")
public class WorkAttendanceController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private WorkAttendanceService workAttendanceService;

    @Autowired
    private CompanyInfoService companyInfoService;

    @Autowired
    private PersonInfoService personInfoService;

    @Autowired
    private AlarmConfigService alarmConfigService;

    @Autowired
    private UnmeasureTemperatureAlarmTask unmeasureTemperatureAlarmTask;

    @Autowired
    private OSSConfig ossConfig;

    @Autowired
    private ValueOperations<String, Object> valueOperations;

    private Map<String,Object> pageSearch(String companyId,String name,
                                        String position1,String position2,
                                        String position3,String position4,
                                        String position5,
                                        Date startDate,Date endDate,
                                        int pageIndex,int pageSize) throws Exception{
        Map<String,Object> dataMap = new HashMap<>();

        if (startDate==null){
            startDate = DateTime.now().dayOfMonth().withMinimumValue().toDate();
        }

        if (endDate==null){
            endDate = DateTime.now().plusMonths(1).dayOfMonth().withMinimumValue().toDate();
        }

        //todo 起始时间与结束时间间隔最大1个月
        DateTime startTime = new DateTime(startDate);
        DateTime endTime = new DateTime(endDate);

        int months = Months.monthsBetween(startTime,endTime).getMonths();

        if(months>1){
            throw new Exception("最多只能查询一个月内的考勤记录!");
        }

        CompanyInfo companyInfo = companyInfoService.get(companyId);

        //todo 查询考勤设置
        List<AlarmConfig> alarmConfigList = alarmConfigService.findByCompanyId(companyId);

        int dayOfWorkAttendanceNum = alarmConfigList.size();

        //todo 查询单位人员

        Map<String,Object> searchParams = new HashMap<>();

        if (StringUtils.isNotEmpty(name)) {
            String[] names = name.split(",");
            List nameList = new ArrayList();
            for (String n : names) {
                nameList.add("%"+ n +"%");
            }
            searchParams.put("nameList", nameList);
        }

        if (StringUtils.isNotEmpty(position1)) {
            searchParams.put("position1", "%" + position1 + "%");
        }

        if (StringUtils.isNotEmpty(position2)) {
            searchParams.put("position2", "%" + position2 + "%");
        }

        if (StringUtils.isNotEmpty(position3)) {
            searchParams.put("position3", "%" + position3 + "%");
        }

        if (StringUtils.isNotEmpty(position4)) {
            searchParams.put("position4", "%" + position4 + "%");
        }

        if (StringUtils.isNotEmpty(position5)) {
            searchParams.put("position5", "%" + position5 + "%");
        }

        searchParams.put("companyCode",companyInfo.getCode() + "%");

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("position1_","asc"));
        sortList.add(new Sort("position2_","asc"));
        sortList.add(new Sort("position3_","asc"));

        Page<PersonInfo> personInfoPage = personInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        int days = Days.daysBetween(startTime,endTime).getDays();

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

        List<Map> personMapList = new ArrayList<>();

        for (PersonInfo personInfo : personInfoPage) {
            //todo 每个人在指定时间段（出勤天数，休息天数，迟到次数，早退次数，上班缺卡次数，下班缺卡次数，旷工天数）
            int workDays=0,restDays=0,lateNum=0,leaveNum=0,missCardOnWorkCount=0,missCardOffWorkCount=0,missCardAllDayCount=0;

            Map<String,Object> personMap = new HashMap<>();
            Map<String,List> workAttendanceMap = new HashMap<>();

            for (int i=0;i<days;i++){
                List<WorkAttendance> list = new ArrayList<>();
                DateTime dt = startTime.plusDays(i);
                workAttendanceMap.put(dt.toString("yyyy-MM-dd"),list);
            }

            //todo 每个人在指定时间段内每天的考勤情况
            List<WorkAttendance> workAttendanceList = workAttendanceService.findByPersonIdAndDate(personInfo.getId(),startDate,endDate);

            Map<String,Boolean> configMap = new HashMap<>();
            for (WorkAttendance workAttendance : workAttendanceList) {
                if(workAttendance.getRecordTime()!=null) {
                    String key = sdf.format(workAttendance.getRecordTime());

                    workAttendanceMap.get(key).add(workAttendance);
                }
            }

            for (int i=0;i<days;i++){
                DateTime dt = startTime.plusDays(i);
                String key = dt.toString("yyyy-MM-dd");

                List<WorkAttendance> workAttendances = workAttendanceMap.get(key);

                if (workAttendances.size()==0){
                    //单天不需要考勤则不会有记录
                    restDays++;
                }
                else{
                    workDays++;

                    for (WorkAttendance workAttendance : workAttendances) {
                        String result = workAttendance.getResult();

                        if (result.equals(WorkAttendance.LATE)){
                            lateNum++; //迟到
                        }
                        else if(result.equals(WorkAttendance.LEAVE_EARLY)){
                            leaveNum++; //早退
                        }
                        else if(result.equals(WorkAttendance.MISSING)){
                            if (workAttendance.getClassifier().equals("1")){
                                //上班缺卡
                                missCardOnWorkCount++;
                            }
                            else{
                                //下班缺卡
                                missCardOffWorkCount++;
                            }
                        }

                        if (dayOfWorkAttendanceNum == missCardOnWorkCount + missCardOffWorkCount){
                            //当天的考勤总数=上下班缺卡之和
                            missCardAllDayCount++;
                        }
                    }
                }
            }

            personMap.put("name",personInfo.getName());
            personMap.put("position1",personInfo.getPosition1());
            personMap.put("position2",personInfo.getPosition2());
            personMap.put("position3",personInfo.getPosition3());
            personMap.put("position4",personInfo.getPosition4());
            personMap.put("position5",personInfo.getPosition5());
            personMap.put("workDays",workDays);
            personMap.put("restDays",restDays);
            personMap.put("lateNum",lateNum);
            personMap.put("leaveNum",leaveNum);
            personMap.put("missCardOnWorkCount",missCardOnWorkCount);
            personMap.put("missCardOffWorkCount",missCardOffWorkCount);
            personMap.put("missCardAllDayCount",missCardAllDayCount);
            personMap.put("workAttendanceMap",workAttendanceMap);

            personMapList.add(personMap);
        }

        dataMap.put("totalElements",personInfoPage.getTotal());
        dataMap.put("totalPage",personInfoPage.getPages());
        dataMap.put("pageNumber",personInfoPage.getPageNum());
        dataMap.put("pageSize",personInfoPage.getPageSize());
        dataMap.put("data", personMapList);

        List<Map> dayColumns = new ArrayList<>();

        for (int i=0;i<days;i++){
            Map<String,Object> map = new HashMap<>();
            DateTime dt = startTime.plusDays(i);

            map.put("name",dt.toString("yyyy-MM-dd"));
            map.put("label",dt.toString("MM-dd") + " " + dt.dayOfWeek().getAsShortText(Locale.CHINA));

            dayColumns.add(map);
        }

        dataMap.put("dayColumns", dayColumns);

        return dataMap;
    }

    @ApiOperation(value="考勤统计列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name="companyId",value = "单位编号",required = true,paramType = "form"),
            @ApiImplicitParam(name="name",value = "用户姓名",required = true,paramType = "form"),
            @ApiImplicitParam(name = "startDate",value = "开始时间", required = true,paramType="form"),
            @ApiImplicitParam(name = "endDate",value = "截止时间", required = true,paramType="form"),
            @ApiImplicitParam(name = "subject",value = "subject", required = false,paramType="form")
    })
    @RequestMapping(value = "statList",method = RequestMethod.POST)
    public MessageResult<Map> statList(
            String companyId,String name,
            String position1,String position2,
            String position3,String position4,
            String position5,
            @DateTimeFormat(pattern = "yyyy-MM-dd") Date startDate,
            @DateTimeFormat(pattern = "yyyy-MM-dd") Date endDate,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){
        MessageResult<Map> msgResult = new MessageResult<>();

        try {
            Map<String,Object> dataMap = pageSearch(companyId,name,position1,position2,position3,position4,position5,startDate,endDate,pageIndex,pageSize);

            msgResult.setData(dataMap);
            msgResult.setResult(true);
        }
        catch (Exception ex){
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
            logger.error(ex.getMessage(),ex);
        }

        return msgResult;
    }

    private CellStyle createCellStyle(Workbook wb){
        CellStyle cellStyle = wb.createCellStyle();

        cellStyle.setBorderTop(BorderStyle.THIN);
        cellStyle.setBorderBottom(BorderStyle.THIN);
        cellStyle.setBorderLeft(BorderStyle.THIN);
        cellStyle.setBorderRight(BorderStyle.THIN);
        cellStyle.setWrapText(true);
        cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);

        return cellStyle;
    }

    @ApiOperation(value="导出考勤报表")
    @RequestMapping(value = "exportXls",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="companyId",value = "单位编号",required = true,paramType = "form"),
            @ApiImplicitParam(name="name",value = "用户姓名",required = true,paramType = "form"),
            @ApiImplicitParam(name="jobNumber",value = "工号",required = true,paramType = "form"),
            @ApiImplicitParam(name = "startDate",value = "开始时间", required = true,paramType="form"),
            @ApiImplicitParam(name = "endDate",value = "截止时间", required = true,paramType="form"),
            @ApiImplicitParam(name = "subject",value = "subject", required = false,paramType="form")
    })
    public MessageResult<String> exportXls(
            String companyId,String name,String jobNumber,
            String position1,String position2,
            String position3,String position4,
            String position5,
            @DateTimeFormat(pattern = "yyyy-MM-dd") Date startDate,
            @DateTimeFormat(pattern = "yyyy-MM-dd") Date endDate,
            @RequestAttribute String subject) throws Exception {
        MessageResult<String> msgResult = new MessageResult<>();

        try {
            DateTime startTime = new DateTime(startDate);
            DateTime endTime = new DateTime(endDate);

            Map<String,Object> dataMap = pageSearch(companyId,name,position1,position2,position3,position4,position5,startDate,endDate,1,10000);

            List<Map> personMapList = (List<Map>)dataMap.get("data");

            int days = Days.daysBetween(startTime, endTime).getDays();
            ClassPathResource resource = new ClassPathResource("static/workAttendanceReport.xls");

            Workbook wb = WorkbookFactory.create(resource.getInputStream());

            Sheet sheet = wb.getSheetAt(0);

            Row row1 = sheet.getRow(0);
            Row row2 = sheet.getRow(1);
            Row row3 = sheet.getRow(2);

            String title = row1.getCell(0).getStringCellValue();
            String subTitle = row2.getCell(0).getStringCellValue();

            row1.getCell(0).setCellValue(String.format(title, startTime.toString("yyyy-MM-dd"), endTime.toString("yyyy-MM-dd")));
            row2.getCell(0).setCellValue(String.format(subTitle, DateTime.now().toString("yyyy-MM-dd HH:mm")));

            int dayStartColIndex = 11;

            for (int i = 0; i < days; i++) {
                DateTime dt = startTime.plusDays(i);
                row3.getCell(dayStartColIndex + i).setCellValue(dt.toString("MM-dd") + "\r\n" + dt.dayOfWeek().getAsShortText(Locale.CHINA));
            }

            //todo 写考勤记录
            int startRowIndex = 3;

            Font redFont = wb.createFont();
            redFont.setColor(Font.COLOR_RED);

            for (int i = 0; i < personMapList.size(); i++) {
                Row row = sheet.createRow(startRowIndex + i);
                Map personMap = personMapList.get(i);

                String personName = (String) personMap.get("name");
                row.createCell(0).setCellValue(personName);

                String _position1 = (String) personMap.get("position1");
                row.createCell(1).setCellValue(_position1);

                String _position2 = (String) personMap.get("position2");
                row.createCell(2).setCellValue(_position2);

                String _position3 = (String) personMap.get("position3");
                row.createCell(3).setCellValue(_position3);

                Integer workDays = Integer.valueOf(personMap.get("workDays").toString());
                row.createCell(4).setCellValue(workDays);

                Integer restDays = Integer.valueOf(personMap.get("restDays").toString());
                row.createCell(5).setCellValue(restDays);

                Integer lateNum = Integer.valueOf(personMap.get("lateNum").toString());
                row.createCell(6).setCellValue(lateNum);

                Integer leaveNum = Integer.valueOf(personMap.get("leaveNum").toString());
                row.createCell(7).setCellValue(leaveNum);

                Integer missCardOnWorkCount = Integer.valueOf(personMap.get("missCardOnWorkCount").toString());
                row.createCell(8).setCellValue(missCardOnWorkCount);

                Integer missCardOffWorkCount = Integer.valueOf(personMap.get("missCardOffWorkCount").toString());
                row.createCell(9).setCellValue(missCardOffWorkCount);

                Integer missCardAllDayCount = Integer.valueOf(personMap.get("missCardAllDayCount").toString());
                row.createCell(10).setCellValue(missCardAllDayCount);

                Map<String, List> workAttendanceMap = (Map<String, List>) personMap.get("workAttendanceMap");

                SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");

                for (int j = 0; j < days; j++) {
                    String key = startTime.plusDays(j).toString("yyyy-MM-dd");

                    if (workAttendanceMap.containsKey(key)) {
                        List<WorkAttendance> workAttendanceList = workAttendanceMap.get(key);

                        List<Map> posList = new ArrayList<>();

                        StringBuilder sb = new StringBuilder();

                        for (WorkAttendance item : workAttendanceList) {
                            if (sb.length() != 0) {
                                sb.append("\r\n");
                            }

                            if (item.getClassifier().equals(1)) {
                                sb.append("上班");
                            } else {
                                sb.append("下班");
                            }

                            if (WorkAttendance.SUCCESS.equals(item.getResult())) {
                                sb.append("打卡");
                            } else if (WorkAttendance.LATE.equals(item.getResult())) {
                                sb.append("迟到");
                            } else if (WorkAttendance.LEAVE_EARLY.equals(item.getResult())) {
                                sb.append("早退");
                            }

                            Map<String, Integer> pos = null;

                            if (WorkAttendance.MISSING.equals(item.getResult())) {
                                pos = new HashMap<>();
                                pos.put("start", sb.length() - 2);
                                sb.append("缺卡");
                            }

                            if (item.getRecordTime() != null) {
                                sb.append(" " + sdf.format(item.getRecordTime()));

                                if (pos != null) {
                                    pos.put("end", sb.length());
                                    posList.add(pos);
                                }
                            }
                        }

                        HSSFRichTextString sText = new HSSFRichTextString(sb.toString());

                        for (Map map : posList) {
                            int start = Integer.valueOf(map.get("start").toString());
                            int end = Integer.valueOf(map.get("end").toString());

                            sText.applyFont(start, end, redFont);
                        }

                        row.createCell(dayStartColIndex + j).setCellValue(sText);
                    }
                }
            }

            CellStyle cellStyle1 = createCellStyle(wb);

            for (int i = 0; i < personMapList.size(); i++) {
                Row row = sheet.getRow(startRowIndex + i);

                if (row != null) {
                    for (int j = 0; j < row.getLastCellNum(); j++) {
                        if (row.getCell(j) != null) {
                            row.getCell(j).setCellStyle(cellStyle1);
                        }
                    }
                }
            }

            //todo 将wb保存到oss
            ByteArrayOutputStream output = new ByteArrayOutputStream();
            wb.write(output);

            byte[] buffer = output.toByteArray();
            ByteArrayInputStream input = new ByteArrayInputStream(buffer);

            String fileName = "考勤统计表.xls";

            String downloadUrl = OSSUtil.upload(ossConfig, "workAttendance", fileName, input);

            wb.write(output);
            wb.close();
            output.close();

            msgResult.setData(downloadUrl);
            msgResult.setResult(true);
        }
        catch (Exception ex){
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
            logger.error(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新考勤记录")
    @ApiImplicitParams({
            @ApiImplicitParam(name="companyId",value = "单位编号",required = true,paramType = "form"),
            @ApiImplicitParam(name = "startDate",value = "开始时间", required = true,paramType="form"),
            @ApiImplicitParam(name = "endDate",value = "截止时间", required = true,paramType="form"),
            @ApiImplicitParam(name = "subject",value = "subject", required = false,paramType="form")
    })
    @RequestMapping(value = "batchUpdate",method = RequestMethod.POST)
    public MessageResult<String> batchUpdate(
            String companyId,
            @DateTimeFormat(pattern = "yyyy-MM-dd") Date startDate,
            @DateTimeFormat(pattern = "yyyy-MM-dd") Date endDate,
            @RequestAttribute String subject) {
        MessageResult<String> messageResult = new MessageResult<>();

        try {
            if (startDate==null){
                startDate = DateTime.now().dayOfMonth().withMinimumValue().toDate();
            }

            if (endDate==null){
                endDate = DateTime.now().plusMonths(1).dayOfMonth().withMinimumValue().toDate();
            }

            //todo 起始时间与结束时间间隔最大1个月
            DateTime startTime = new DateTime(startDate);
            DateTime endTime = new DateTime(endDate);

            int months = Months.monthsBetween(startTime,endTime).getMonths();

            if(months>1){
                throw new Exception("最多只能更新一个月内的考勤记录!");
            }

            int affectCount = unmeasureTemperatureAlarmTask.batchUpdate(companyId, startDate, endDate);

            messageResult.setResult(true);
            messageResult.setMessage("共添加" + affectCount + "考勤记录");
        }
        catch (Exception ex){
            logger.error(ex.getMessage(),ex);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @ApiOperation(value="微信通知")
    @ApiImplicitParams({
            @ApiImplicitParam(name="alarmConfigId",value = "单位编号",required = true,paramType = "form")
    })
    @RequestMapping(value = "wxNotice",method = RequestMethod.POST)
    public MessageResult<String> wxNotice(String alarmConfigId) {
        MessageResult<String> messageResult = new MessageResult<>();

        try {
            AlarmConfig alarmConfig = alarmConfigService.get(alarmConfigId);

            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");
            String date = DateTime.now().toString("yyyy-MM-dd");

            DateTime startTime = new DateTime(sdf.parse(date + " " + alarmConfig.getStartTime()));
            DateTime endTime = new DateTime(sdf.parse(date + " " + alarmConfig.getEndTime()));

            unmeasureTemperatureAlarmTask.noticePerson(alarmConfig,startTime,endTime);
            unmeasureTemperatureAlarmTask.noticeManager(alarmConfig,startTime,endTime);

            messageResult.setResult(true);
        }
        catch (Exception ex){
            logger.error(ex.getMessage(),ex);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @ApiOperation(value="添加考勤记录")
    @ApiImplicitParams({
            @ApiImplicitParam(name="personId",value = "人员编号",required = true,paramType = "form"),
            @ApiImplicitParam(name="temperature",value = "温度",required = true,paramType = "form"),
            @ApiImplicitParam(name = "recordTime",value = "考勤时间", required = true,paramType="form")
    })
    @RequestMapping(value = "add",method = RequestMethod.POST)
    public MessageResult<String> add(
            Long personId, BigDecimal temperature,
            @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss") Date recordTime) {
        MessageResult<String> messageResult = new MessageResult<>();

        try {
            boolean result = workAttendanceService.punchIn(personId,temperature,recordTime);

            messageResult.setResult(result);

            messageResult.setMessage(result ? "添加考勤记录成功!" : "添加考勤记录失败!");
        }
        catch (Exception ex){
            logger.error(ex.getMessage(),ex);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }
}
