package com.jpsoft.smart.modules.mobile.controller;

import com.alibaba.fastjson.JSONObject;
import com.alipay.api.domain.Person;
import com.github.pagehelper.Page;
import com.jpsoft.smart.config.OSSConfig;
import com.jpsoft.smart.config.RedisConfig;
import com.jpsoft.smart.modules.base.entity.CompanyInfo;
import com.jpsoft.smart.modules.base.entity.PersonCompany;
import com.jpsoft.smart.modules.base.entity.PersonDeviceRelation;
import com.jpsoft.smart.modules.base.entity.PersonInfo;
import com.jpsoft.smart.modules.base.service.CompanyInfoService;
import com.jpsoft.smart.modules.base.service.PersonCompanyService;
import com.jpsoft.smart.modules.base.service.PersonDeviceRelationService;
import com.jpsoft.smart.modules.base.service.PersonInfoService;
import com.jpsoft.smart.modules.common.dto.MessageResult;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.modules.common.utils.*;
import com.jpsoft.smart.modules.lapi.service.ILapiService;
import com.jpsoft.smart.modules.lapi.vo.LapiMsgResult;
import com.jpsoft.smart.modules.sys.entity.User;
import com.jpsoft.smart.modules.sys.service.UserService;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.security.Keys;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.formula.functions.T;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.security.Key;
import java.util.*;
import java.util.concurrent.TimeUnit;

@RestController
@RequestMapping("/mobile/personInfoApi")
public class PersonInfoApiController {
    @Value("${jwt.secret}")
    private String jwtSecret;

    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private OSSConfig ossConfig;

    @Autowired
    private PersonInfoService personInfoService;

    @Autowired
    private PersonDeviceRelationService personDeviceRelationService;

    @Autowired
    private ILapiService lapiService;

    @Autowired
    private ValueOperations<String, Object> valueOperations;

    @Autowired
    private UserService userService;

    @Autowired
    private CompanyInfoService companyInfoService;

    @Autowired
    private PersonCompanyService personCompanyService;

    @Autowired
    RabbitTemplate rabbitTemplate;

    @PostMapping("findByNameAndPhone")
    @ApiOperation(value="通过姓名和手机号查询人员(公开接口)")
    @ApiImplicitParams({
            @ApiImplicitParam(name="name",value = "姓名",required = true,paramType = "form"),
            @ApiImplicitParam(name = "phone",value = "电话号码", required = true,paramType="form")
    })
    public MessageResult<Long> findByNameAndPhone(String name,String phone){
        MessageResult<Long> messageResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.findByNameAndPhone(name, phone);

            if (personInfo==null){
                throw new Exception("您的信息暂未登记，请先联系管理人员!");
            }

            if (personInfo.getFaceEnabled()==null || !personInfo.getFaceEnabled()){
                throw new Exception("当前用户未被允许使用人脸识别!");
            }

            messageResult.setData(personInfo.getId());
            messageResult.setResult(true);
            messageResult.setCode(200);
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("switchCompany")
    @ApiOperation(value="切换管理单位")
    @ApiImplicitParams({
            @ApiImplicitParam(name="companyId",value = "companyId",required = true,paramType = "form"),
            @ApiImplicitParam(name="token",value = "令牌",paramType = "form"),
            @ApiImplicitParam(name="subject",value = "目标(不传)",paramType = "form")
    })
    public MessageResult<Map> switchCompany(String companyId,String token,@RequestAttribute String subject){
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.findById(Long.valueOf(subject));

            if (personInfo==null){
                throw new Exception("当前用户不存在!");
            }

            //判断companyId是否存在于当前用户的关联单位中
            List<PersonCompany> personCompanies = personCompanyService.findByPersonId(personInfo.getId());

            long count = personCompanies.stream().filter((item)->item.getCompanyId().equals(companyId)).count();

            if (count==0) {
                //当前用户可以管理本单位
                if (personInfo.getAllowViewLocal() && personInfo.getCompanyId().equals(companyId)) {
                    count = 1;
                }
            }

            if (count==0){
                throw new Exception("选择单位不在当前用户的管理范围内!");
            }

            CompanyInfo companyInfo = companyInfoService.get(companyId);

            Map<String,Object> companyMap = new HashMap<>();
            companyMap.put("id",companyInfo.getId());
            companyMap.put("name",companyInfo.getName());

            valueOperations.set(RedisConfig.PERSON_SELECT_COMPANY + personInfo.getId(), companyMap,2,TimeUnit.HOURS);

            messageResult.setData(companyMap);
            messageResult.setResult(true);
            messageResult.setCode(200);
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("findByOpenId")
    @ApiOperation(value="通过openId查询人员(公开接口)")
    @ApiImplicitParams({
            @ApiImplicitParam(name="openId",value = "微信openId",required = true,paramType = "form")
    })
    public MessageResult<Map> findByOpenId(String openId){
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.findByOpenId(openId);

            if (personInfo==null){
                throw new Exception("当前用户不存在!");
            }

            Map<String,Object> dataMap = new HashMap<String, Object>();
            String token = JwtUtil.createToken(jwtSecret,personInfo.getId() +"", DateTime.now().plusHours(6).toDate());

            dataMap.put("person",personInfo);
            dataMap.put("token", token);

            CompanyInfo selectedCompany = new CompanyInfo();

            valueOperations.get(RedisConfig.PERSON_SELECT_COMPANY + personInfo.getId());

            Map<String,Object> companyMap = (Map<String, Object>) valueOperations.get(RedisConfig.PERSON_SELECT_COMPANY + personInfo.getId());

            if (companyMap==null) {
                companyMap = new HashMap<>();

                //当前用户默认管理单位
                if (StringUtils.isNotEmpty(personInfo.getPopedom()) && personInfo.getPopedom().indexOf("2") != -1) {
                    if (personInfo.getAllowViewLocal()) {
                        CompanyInfo companyInfo = companyInfoService.get(personInfo.getCompanyId());
                        selectedCompany = companyInfo;
                    } else {
                        List<PersonCompany> personCompanies = personCompanyService.findByPersonId(personInfo.getId());

                        if (personCompanies.size() > 0) {
                            CompanyInfo companyInfo = companyInfoService.get(personCompanies.get(0).getCompanyId());

                            if(companyInfo!=null) {
                                selectedCompany = companyInfo;
                            }
                        }
                    }
                }

                if(selectedCompany!=null) {
                    companyMap.put("id", selectedCompany.getId());
                    companyMap.put("name", selectedCompany.getName());

                    valueOperations.set(RedisConfig.PERSON_SELECT_COMPANY + personInfo.getId(), companyMap, 2, TimeUnit.HOURS);
                }
            }

            dataMap.put("company",companyMap);

            messageResult.setData(dataMap);
            messageResult.setResult(true);
            messageResult.setCode(200);
        }
        catch (Exception ex){
            logger.error(ex.getMessage(),ex);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("getVerifyCode")
    @ApiOperation(value="获取短信验证码(公开接口)")
    @ApiImplicitParams({
            @ApiImplicitParam(name="personId",value = "人员编号",required = true,paramType = "form")
    })
    public MessageResult<String> getVerifyCode(Long personId){
        MessageResult<String> messageResult = new MessageResult<>();

        try {
             PersonInfo personInfo = personInfoService.get(personId);

            if(personInfo==null){
                throw new Exception("人员编号不存在!");
            }

            String key = "SMS_" + personId;

            String  verifyCode =  (String)valueOperations.get(key);

            if(StringUtils.isEmpty(verifyCode)) {
                verifyCode = SMSUtil.generateNumberString(6);
                JSONObject verifyCodeJSON = new JSONObject();
                verifyCodeJSON.put("code", verifyCode);

                messageResult = SMSUtil.send(personInfo.getPhone(), "SMS_49390047", verifyCodeJSON.toString());

                //设置当前用户的验证码,1分钟内有效
                valueOperations.set(key, verifyCode, 1, TimeUnit.MINUTES);
            }

            messageResult.setResult(true);
            messageResult.setCode(200);
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("validateCode")
    @ApiOperation(value="验证短信验证码(公开接口)")
    @ApiImplicitParams({
            @ApiImplicitParam(name="personId",value = "人员编号",required = true,paramType = "form"),
            @ApiImplicitParam(name="verifyCode",value = "验证码",required = true,paramType = "form"),
            @ApiImplicitParam(name="openId",value = "微信openId",paramType = "form")
    })
    public MessageResult<String> validateCode(Long personId,String verifyCode,String openId){
        MessageResult<String> messageResult = new MessageResult<>();

        try {
            String smsKey = "SMS_" + personId;

            String  beforeVerifyCode = (String)valueOperations.get(smsKey);

            if(StringUtils.isEmpty(beforeVerifyCode)) {
                throw new Exception("验证码已过期!");
            }

            if (!beforeVerifyCode.equals(verifyCode)){
                throw new Exception("验证码错误!");
            }

            if (StringUtils.isNotEmpty(openId)){
                PersonInfo personInfo = personInfoService.get(personId);
                personInfo.setOpenId(openId);
                personInfo.setUpdateTime(new Date());

                personInfoService.update(personInfo);
            }

            String token = JwtUtil.createToken(jwtSecret,String.valueOf(personId), DateTime.now().plusHours(6).toDate());

            messageResult.setData(token);
            messageResult.setResult(true);
            messageResult.setCode(200);
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("upload")
    @ApiOperation(value="人员照片上传")
    @ApiImplicitParams({
            @ApiImplicitParam(name="photoName",value = "照片名称",required = true,paramType = "form"),
            @ApiImplicitParam(name = "photoFile",value = "员工照片", required = true,paramType="form", dataType = "__file"),
            @ApiImplicitParam(name="token",value = "令牌",required = false,paramType = "query"),
            @ApiImplicitParam(name="subject",value = "令牌",required = false,paramType = "query")
    })
    public MessageResult<String> upload(
            String photoName, MultipartFile photoFile,String token,@RequestAttribute  String subject){
        MessageResult<String> messageResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(Long.valueOf(subject));

            if (personInfo==    null){
                throw new Exception("人员信息不存在!");
            }

//            if (personInfo.getFaceBound()!=null && personInfo.getFaceBound()){
//                throw new Exception("人员照片已绑定到设备，不能再上传!");
//            }

            String retFileUrl = OSSUtil.upload(ossConfig,"/person",photoName,photoFile.getInputStream());

            messageResult.setResult(true);
            messageResult.setData(retFileUrl);
            messageResult.setCode(200);
        } catch (Exception e) {
            logger.error(e.getMessage(),e);

            messageResult.setResult(false);
            messageResult.setMessage(e.getMessage());
        }

        return messageResult;
    }

    @PostMapping("save")
    @ApiOperation(value="保存人员信息并将照片上传到终端")
    @ApiImplicitParams({
            @ApiImplicitParam(name="personId",value = "人员编号",required = true,paramType = "form"),
            @ApiImplicitParam(name="openId",value = "微信openId",required = true,paramType = "form"),
            @ApiImplicitParam(name="faceImageUrl",value = "照片地址",required = true,paramType = "form"),
            @ApiImplicitParam(name="token",value = "令牌",paramType = "form")
    })
    public MessageResult<PersonInfo> save(Long personId,String openId,String faceImageUrl,String token){
        MessageResult<PersonInfo> messageResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(personId);

            if (personInfo==null){
                throw new Exception("人员信息不存在!");
            }

            if (personInfo.getFaceBound()!=null && personInfo.getFaceBound()){
                throw new Exception("人员照片已绑定到设备，不能再上传!");
            }

            //todo 更新人脸照片
            personInfo.setOpenId(openId);
            personInfo.setFaceImageUrl(faceImageUrl);
            personInfo.setUpdateTime(new Date());

            personInfoService.update(personInfo);

            //todo 加入到导入图片队列中
            rabbitTemplate.convertAndSend("importImageExchange","importImage",personId);

            messageResult.setData(personInfo);
            messageResult.setResult(true);

            messageResult.setCode(200);
        } catch (Exception e) {
            logger.error(e.getMessage(),e);

            messageResult.setResult(false);
            messageResult.setMessage(e.getMessage());
        }

        return messageResult;
    }

    @ApiOperation(value="人员列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "name",value = "姓名", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "idCard",value = "身份证", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "phone",value = "手机号", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name="token",value = "令牌",paramType = "query",dataType = "form"),
            @ApiImplicitParam(name="subject",value = "目标(不传)",paramType = "form")
    })
    public MessageResult<Map> pageList(
            @RequestParam(value="name",defaultValue="") String name,
            @RequestParam(value="idCard",defaultValue="") String idCard,
            @RequestParam(value="phone",defaultValue="") String phone,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            String token,
            @RequestAttribute String subject){
        MessageResult<Map> msgResult = new MessageResult<>();

        try {
            User user = userService.get(subject);

            if (user==null){
                throw new Exception("当前用户不是管理员！");
            }

            Map<String, Object> searchParams = new HashMap<>();

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("a.create_time", "asc"));

            if (StringUtils.isNotEmpty(name)) {
                searchParams.put("name", "%" + name + "%");
            }
            if (StringUtils.isNotEmpty(idCard)) {
                searchParams.put("idCard", "%" + idCard + "%");
            }
            if (StringUtils.isNotEmpty(phone)) {
                searchParams.put("phone", "%" + phone + "%");
            }

            searchParams.put("companyId", user.getCompanyId());

            Page<PersonInfo> page = personInfoService.pageSearch(searchParams, pageIndex, pageSize, true, sortList);

            msgResult.setResult(true);
            msgResult.setData(PojoUtils.pageWrapper(page));
        }
        catch (Exception e) {
            logger.error(e.getMessage(),e);
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
        }



        return msgResult;
    }

    @PostMapping("add")
    @ApiOperation(value="添加人员信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name="name",value = "姓名",required = true,paramType = "form"),
            @ApiImplicitParam(name="phone",value = "电话",required = true,paramType = "form"),
            @ApiImplicitParam(name="faceImageUrl",value = "照片地址",required = true,paramType = "form"),
            @ApiImplicitParam(name="bindDevices",value = "绑定设备(,分隔)",required = true,paramType = "form"),
            @ApiImplicitParam(name="token",value = "令牌",paramType = "form"),
            @ApiImplicitParam(name="subject",value = "目标(不传)",paramType = "form")
    })
    public MessageResult<PersonInfo> add(String name,String phone,String faceImageUrl,String bindDevices,String token,@RequestAttribute String subject){
        MessageResult<PersonInfo> messageResult = new MessageResult<>();

        try {
            PersonInfo admin = personInfoService.get(Long.valueOf(subject));

            Map<String,Object> companyMap = (Map<String,Object>)valueOperations.get(RedisConfig.PERSON_SELECT_COMPANY + admin.getId());
            String companyId = companyMap.get("id").toString();

            if (StringUtils.isEmpty(admin.getPopedom())
            && admin.getPopedom().indexOf("2")==-1){
                throw new Exception("当前用户不是管理员！");
            }

            PersonInfo personInfo = personInfoService.findByNameAndPhone(name, phone);

            if(personInfo == null) {
                personInfo = new PersonInfo();
                personInfo.setCompanyId(companyId);
                personInfo.setName(name);
                personInfo.setPhone(phone);
                personInfo.setFaceEnabled(true);
                personInfo.setFaceImageUrl(faceImageUrl);
                personInfo.setPopedom("1");
                personInfo.setDelFlag(false);
                personInfo.setCreateBy(String.valueOf(admin.getId()));
                personInfo.setCreateTime(new Date());

                //插入后，会将自增id写到id属性中
                int affectCount = personInfoService.insert(personInfo);

//                //todo 关联设备
//                String[] deviceIds = bindDevices.split(",");
//                for(String deviceId : deviceIds){
//                    PersonDeviceRelation personDeviceRelation = new PersonDeviceRelation();
//                    personDeviceRelation.setId(UUID.randomUUID().toString());
//                    personDeviceRelation.setDeviceId(deviceId);
//                    personDeviceRelation.setPersonId(personInfo.getId());
//                    personDeviceRelation.setDelFlag(false);
//                    personDeviceRelation.setCreateBy(subject);
//                    personDeviceRelation.setCreateTime(new Date());
//                    personDeviceRelationService.insert(personDeviceRelation);
//                }

                if(affectCount>0) {
                    //todo 加入到导入图片队列中
                    rabbitTemplate.convertAndSend("importImageExchange", "importImage", personInfo.getId());
                }

                messageResult.setData(personInfo);
                messageResult.setResult(true);
                messageResult.setCode(200);
            }
            else{
                messageResult.setResult(false);
                messageResult.setMessage("人员已存在。");
            }
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
            messageResult.setResult(false);
            messageResult.setMessage(e.getMessage());
        }

        return messageResult;
    }

    @ApiOperation(value="保存人员信息并将照片上传到终端")
    @PostMapping("saveDetail")
    @ApiImplicitParams({
            @ApiImplicitParam(name="name",value = "姓名",required = false,paramType = "form"),
            @ApiImplicitParam(name="phone",value = "手机号码",required = false,paramType = "form"),
            @ApiImplicitParam(name="idCard",value = "身份证号",required = false,paramType = "form"),
            @ApiImplicitParam(name="wechatNoticeEnabled",value = "是否接收微信通知",required = false,paramType = "form",dataType = "Boolean"),
            @ApiImplicitParam(name="openId",value = "微信openId",required = true,paramType = "form"),
            @ApiImplicitParam(name="token",value = "令牌",paramType = "form")
    })
    public MessageResult<PersonInfo> saveDetail(String name,String phone,String idCard,Boolean wechatNoticeEnabled,String openId,String token){
        MessageResult<PersonInfo> messageResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.findByOpenId(openId);

            if (personInfo==null){
                throw new Exception("人员信息不存在!");
            }

            CheckIdCard cic = null;
            if (StringUtils.isEmpty(idCard)) {
//                    msgResult.setMessage("身份证不能为空!");
//                    msgResult.setResult(false);
//                    return msgResult;
            } else {
                personInfo.setIdCard(idCard.trim().toUpperCase());

                cic = new CheckIdCard(personInfo.getIdCard());
                if (!cic.validate()) {
                    messageResult.setMessage("身份证验证失败!");
                    messageResult.setResult(false);
                    return messageResult;
                }
            }

            personInfo.setName(name);
            personInfo.setPhone(phone);
            if(wechatNoticeEnabled != null) personInfo.setWechatNoticeEnabled(wechatNoticeEnabled);
            personInfo.setUpdateTime(new Date());

            personInfoService.update(personInfo);

            messageResult.setData(personInfo);
            messageResult.setResult(true);

            messageResult.setCode(200);
        } catch (Exception e) {
            logger.error(e.getMessage(),e);

            messageResult.setResult(false);
            messageResult.setMessage(e.getMessage());
        }

        return messageResult;
    }
}
