package com.jpsoft.smart.modules.mobile.controller;

import com.github.pagehelper.Page;
import com.jpsoft.smart.config.TemperatureConfig;
import com.jpsoft.smart.modules.base.entity.PersonInfo;
import com.jpsoft.smart.modules.base.service.AlarmConfigService;
import com.jpsoft.smart.modules.base.service.CompanyInfoService;
import com.jpsoft.smart.modules.base.service.PersonInfoService;
import com.jpsoft.smart.modules.business.entity.WorkAttendance;
import com.jpsoft.smart.modules.business.service.WorkAttendanceService;
import com.jpsoft.smart.modules.common.dto.MessageResult;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.modules.common.utils.PojoUtils;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/mobile/workAttendance")
@Slf4j
public class WorkAttendanceApiController {
    @Autowired
    private PersonInfoService personInfoService;

    @Autowired
    private CompanyInfoService companyInfoService;

    @Autowired
    private AlarmConfigService alarmConfigService;

    @Autowired
    private TemperatureConfig temperatureConfig;

    @Autowired
    private WorkAttendanceService workAttendanceService;

    @ApiOperation(value="考勤记录")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="token",value = "令牌",required = true,paramType = "form"),
            @ApiImplicitParam(name="subject",value = "目标(不传)",paramType = "form")
    })
    public MessageResult<Map> pageList(@RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
                                       @RequestParam(value="pageSize",defaultValue="20") int pageSize,
                                       @RequestAttribute String subject, String token){
        MessageResult<Map> msgResult = new MessageResult<>();

        try {
            Map<String, Object> searchParams = new HashMap<>();

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("record_time", "desc"));

            PersonInfo personInfo = personInfoService.get(Long.valueOf(subject));

            searchParams.put("personId", personInfo.getId());

            Page<WorkAttendance> page = workAttendanceService.pageSearch(searchParams, pageIndex, pageSize, true, sortList);

            Map<String,Object> dataMap = PojoUtils.pageWrapper(page);

            msgResult.setData(dataMap);
            msgResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
