package com.jpsoft.smart.modules.pay.alipay;


import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.domain.AlipayTradeCreateModel;
import com.alipay.api.domain.AlipayTradeWapPayModel;
import com.alipay.api.domain.BusinessParams;
import com.alipay.api.domain.ExtendParams;
import com.alipay.api.internal.util.AlipaySignature;
import com.alipay.api.request.AlipaySystemOauthTokenRequest;
import com.alipay.api.request.AlipayTradeCreateRequest;
import com.alipay.api.request.AlipayTradePrecreateRequest;
import com.alipay.api.request.AlipayTradeWapPayRequest;
import com.alipay.api.response.AlipaySystemOauthTokenResponse;
import com.alipay.api.response.AlipayTradeCreateResponse;
import com.alipay.api.response.AlipayTradePrecreateResponse;
import com.alipay.api.response.AlipayTradeWapPayResponse;
import com.ijpay.alipay.AliPayApi;
import com.ijpay.alipay.AliPayApiConfig;
import com.ijpay.alipay.AliPayApiConfigKit;
import com.jpsoft.smart.modules.base.entity.CompanyPaymentInfo;
import com.jpsoft.smart.modules.base.entity.ElectricClientInfo;
import com.jpsoft.smart.modules.base.entity.ParkingRecord;
import com.jpsoft.smart.modules.base.entity.RechargeRecord;
import com.jpsoft.smart.modules.base.service.CompanyPaymentInfoService;
import com.jpsoft.smart.modules.base.service.ElectricClientInfoService;
import com.jpsoft.smart.modules.base.service.ParkingRecordService;
import com.jpsoft.smart.modules.base.service.RechargeRecordService;
import com.jpsoft.smart.modules.business.service.RechargeService;
import com.jpsoft.smart.modules.common.dto.MessageResult;
import com.jpsoft.smart.electricity.server.protocol.MeterReceivePacket;
import com.jpsoft.smart.modules.common.service.IRedisService;
import com.jpsoft.smart.modules.common.utils.ParkApiUtil;
import com.jpsoft.smart.modules.pay.properties.AliPayProperties;
import com.jpsoft.smart.modules.pay.properties.ParkAliPayProperties;
import com.jpsoft.smart.modules.wechat.vo.UserInfo;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.annotations.Param;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author 墨鱼_mo
 * @date 2019-11-26 9:48
 */

@Slf4j
@RequestMapping("/aliPay")
@RestController
public class AlipayController {


    @Autowired
    private AliPayProperties aliPayProperties;

    @Autowired
    private ParkAliPayProperties parkAliPayProperties;

    @Autowired
    private RechargeRecordService rechargeRecordService;

    @Autowired
    private RechargeService rechargeService;

    @Autowired
    private ElectricClientInfoService electricClientInfoService;

    @Autowired
    private ParkingRecordService parkingRecordService;

    @Autowired
    private CompanyPaymentInfoService companyPaymentInfoService;

    @Autowired
    private IRedisService redisService;

    @Autowired
    private ParkApiUtil parkApiUtil;



    @ApiOperation(value = "支付宝扫码支付")
    @GetMapping("/tradePrecreatePay")
    public MessageResult tradePrecreatePay(String recordId) {

        MessageResult result = new MessageResult();

        try {
            AlipayClient alipayClient = new DefaultAlipayClient(aliPayProperties.getServiceUrl(), aliPayProperties.getAppId(), aliPayProperties.getPrivateKey(), "json", aliPayProperties.getInputCharset(), aliPayProperties.getZfbPublicKey(), aliPayProperties.getSignType());
            AlipayTradePrecreateRequest request = new AlipayTradePrecreateRequest();
            request.setNotifyUrl(aliPayProperties.getNotifyUrl());

            RechargeRecord rechargeRecord = rechargeRecordService.get(recordId);

            if (rechargeRecord == null) {
                throw new Exception("订单不存在");
            }
            if (rechargeRecord.getBuyAmount().compareTo(BigDecimal.ZERO) != 1) {
                throw new Exception("金额有误");
            }
            if (rechargeRecord.getPaymentStatus().equals("20")) {
                throw new Exception("订单已支付");
            }


            Map<String, Object> maps = new HashMap<>();
            maps.put("out_trade_no", rechargeRecord.getSerialNumber());
            maps.put("total_amount", String.valueOf(rechargeRecord.getBuyAmount()));
            maps.put("subject", rechargeRecord.getProductTheme());
            maps.put("timeout_express", "5m");

           /* JSONObject extendParams = new JSONObject();

            extendParams.put("sys_service_provider_id", aliPayProperties.getMchId());
            maps.put("extend_params", extendParams);
            request.putOtherTextParam("app_auth_token", aliPayProperties.getAppAuthToken());*/
            //把订单信息转换为json对象的字符串
            String postdata = JSONObject.toJSONString(maps);
            request.setBizContent(postdata);
            AlipayTradePrecreateResponse response = alipayClient.execute(request);
            String body = response.getBody();
            JSONObject jsonObject = JSONObject.parseObject(body);
            String alipayTradePrecreateResponse = jsonObject.getString("alipay_trade_precreate_response");
            JSONObject jsonResponse = JSONObject.parseObject(alipayTradePrecreateResponse);

            if (!jsonResponse.getString("code").equals("10000")) {
                throw new Exception(jsonResponse.getString("sub_msg"));
            }

            String qr_code = jsonObject.getJSONObject("alipay_trade_precreate_response").getString("qr_code");
            result.setResult(true);

            log.warn("qr_code=" + qr_code);
            result.setData(qr_code);

        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage(), e);
            result.setResult(false);
            result.setMessage(e.getMessage());
        }
        return result;

    }

    @RequestMapping(value="/payNotify")
    public String payNotify(HttpServletRequest request) {
        try {
            // 获取支付宝POST过来反馈信息
            Map<String, String> params = AliPayApi.toMap(request);

            for (Map.Entry<String, String> entry : params.entrySet()) {
                System.out.println(entry.getKey() + " = " + entry.getValue());
            }

            boolean verifyResult = AlipaySignature.rsaCheckV1(params, aliPayProperties.getZfbPublicKey(), "UTF-8", "RSA2");

            if (verifyResult) {

                RechargeRecord rechargeRecord = rechargeRecordService.getBySerialNum(params.get("out_trade_no"));
                if (rechargeRecord == null) {
                    throw new Exception("支付已完成，但未找到此订单号的流水");
                }
                if (!rechargeRecord.getBuyAmount().toString().equals(params.get("buyer_pay_amount"))) {
                    throw new Exception("支付金额与系统金额不一致");
                }
                rechargeRecord.setUpdateTime(new Date());
                rechargeRecord.setUpdateBy(params.get("buyer_logon_id"));
                rechargeRecord.setBuyType("alipay");
                rechargeRecord.setPaymentStatus("20");
                rechargeRecord.setTransactionNumber(params.get("trade_no"));
                rechargeRecord.setPaymentTime(DateUtil.parse(params.get("gmt_payment")));

                try {
                    MeterReceivePacket mp = rechargeService.recharge(rechargeRecord.getRoomId(), rechargeRecord.getBuyElectricity(), 3, TimeUnit.SECONDS);
                    if (mp == null){
                        rechargeRecord.setErrorLog("相应超时");
                    }else {
                        if (!mp.isValidateResult()){
                            rechargeRecord.setErrorLog("校验失败");
                        }else {
                            rechargeRecord.setChargingStatus("20");
                            rechargeRecord.setDumpEnergy(mp.getRemain()); //当前剩余电量

                            ElectricClientInfo clientInfo = electricClientInfoService.findOneByRoomId(rechargeRecord.getRoomId());

                            if (clientInfo!=null){
                                //记录剩余电量
                                clientInfo.setElectricityRemaining(mp.getRemain());

                                //记录充电次数
                                clientInfo.setRechargeTimes(mp.getRechargeTimes());
                                clientInfo.setActive(mp.getState()!=1);

                                clientInfo.setUpdateTime(new Date());

                                electricClientInfoService.update(clientInfo);
                            }
                        }
                    }
                } catch (Exception e) {
                    rechargeRecord.setErrorLog(e.getMessage());
                    e.printStackTrace();
                    log.error(e.getMessage(), e);
                }
                rechargeRecordService.update(rechargeRecord);


                return "success";
            } else {
                log.error("notify_url 验证失败");
                return "failure";
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage(), e);
            return "failure";
        }
    }

    @ApiOperation(value = "停车支付宝扫码支付")
    @PostMapping("/parkTradePrecreatePay")
    public MessageResult parkTradePrecreatePay(@RequestBody HashMap<String, String> map) {

        MessageResult result = new MessageResult();

        try {
            ParkingRecord parkingRecord = parkingRecordService.get(map.get("id"));
            if (parkingRecord == null){
                throw new Exception("订单不存在");
            }
            CompanyPaymentInfo companyPaymentInfo = companyPaymentInfoService.findByCompanyId(parkingRecord.getCompanyId());
            if (companyPaymentInfo == null){
                throw new Exception("公司未配置收款账户");
            }

            AlipayClient alipayClient = new DefaultAlipayClient(aliPayProperties.getServiceUrl(), aliPayProperties.getAppId(), aliPayProperties.getPrivateKey(), "json", aliPayProperties.getInputCharset(), aliPayProperties.getZfbPublicKey(), aliPayProperties.getSignType());
            AlipayTradePrecreateRequest request = new AlipayTradePrecreateRequest();
            request.setNotifyUrl(companyPaymentInfo.getParkingAlipayNotifyUrl());



            if (parkingRecord.getPayAmount().compareTo(BigDecimal.ZERO) != 1) {
                throw new Exception("金额有误");
            }
            if (parkingRecord.getPayStatus().equals("20")) {
                throw new Exception("订单已支付");
            }


            Map<String, Object> maps = new HashMap<>();
            maps.put("out_trade_no", parkingRecord.getSerialNumber());
            maps.put("total_amount", String.valueOf(parkingRecord.getPayAmount()));
            maps.put("subject", parkingRecord.getProductTheme());
            maps.put("timeout_express", "5m");

            JSONObject extendParams = new JSONObject();

            extendParams.put("sys_service_provider_id", aliPayProperties.getMchId());
            maps.put("extend_params", extendParams);
            request.putOtherTextParam("app_auth_token", companyPaymentInfo.getAppAuthToken());
            //把订单信息转换为json对象的字符串
            String postdata = JSONObject.toJSONString(maps);
            request.setBizContent(postdata);
            AlipayTradePrecreateResponse response = alipayClient.execute(request);
            String body = response.getBody();
            JSONObject jsonObject = JSONObject.parseObject(body);
            String alipayTradePrecreateResponse = jsonObject.getString("alipay_trade_precreate_response");
            JSONObject jsonResponse = JSONObject.parseObject(alipayTradePrecreateResponse);

            if (!jsonResponse.getString("code").equals("10000")) {
                throw new Exception(jsonResponse.getString("sub_msg"));
            }

            String qr_code = jsonObject.getJSONObject("alipay_trade_precreate_response").getString("qr_code");
            result.setResult(true);

            log.warn("qr_code=" + qr_code);
            result.setData(qr_code);

        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage(), e);
            result.setResult(false);
            result.setMessage(e.getMessage());
        }
        return result;

    }


    @ApiOperation(value = "停车支付宝手机网站支付")
    @PostMapping("/parkingTradeWebPay")
    public MessageResult parkingTradeWebPay(@RequestBody HashMap<String, String> map) {

        MessageResult result = new MessageResult();

        try {
            ParkingRecord parkingRecord = parkingRecordService.get(map.get("id"));
            if (parkingRecord == null){
                throw new Exception("订单不存在");
            }
            CompanyPaymentInfo companyPaymentInfo = companyPaymentInfoService.findByCompanyId(parkingRecord.getCompanyId());
            if (companyPaymentInfo == null){
                throw new Exception("公司未配置收款账户");
            }




            AlipayClient alipayClient = new DefaultAlipayClient(parkAliPayProperties.getServiceUrl(), parkAliPayProperties.getAppId(), parkAliPayProperties.getPrivateKey(), "json", parkAliPayProperties.getInputCharset(), parkAliPayProperties.getZfbPublicKey(), parkAliPayProperties.getSignType());
            AlipayTradeCreateRequest request = new AlipayTradeCreateRequest ();
            request.setNotifyUrl(companyPaymentInfo.getParkingAlipayNotifyUrl());



            if (parkingRecord.getPayAmount().compareTo(BigDecimal.ZERO) != 1) {
                throw new Exception("金额有误");
            }
            if (parkingRecord.getPayStatus().equals("20")) {
                throw new Exception("订单已支付");
            }


            Map<String, Object> maps = new HashMap<>();
            maps.put("out_trade_no", parkingRecord.getSerialNumber());
            maps.put("total_amount", String.valueOf(parkingRecord.getPayAmount()));
            maps.put("subject", parkingRecord.getProductTheme());
            maps.put("timeout_express", "5m");
            maps.put("buyer_id", parkingRecord.getOpenId());

            JSONObject extendParams = new JSONObject();

            extendParams.put("sys_service_provider_id", aliPayProperties.getMchId());
            maps.put("extend_params", extendParams);
            request.putOtherTextParam("app_auth_token", companyPaymentInfo.getAppAuthToken());
            //把订单信息转换为json对象的字符串
            String postdata = JSONObject.toJSONString(maps);
            request.setBizContent(postdata);
            AlipayTradeCreateResponse response = alipayClient.execute(request);
            if(response.isSuccess()){

                HashMap<String,Object> dataMap = new HashMap<String,Object>();

                dataMap.put("tradeNo", response.getTradeNo());

                HashMap<String,Object> resultMap = new HashMap<String,Object>();

                resultMap.put("tradeOrder", dataMap);

                result.setData(resultMap);
                result.setResult(true);

            } else {
                result.setResult(false);
                result.setData("");

            }



        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage(), e);
            result.setResult(false);
            result.setMessage(e.getMessage());
        }
        return result;

    }

    @RequestMapping(value="/parkingAlipayNotify")
    public String parkingAlipayNotify(HttpServletRequest request) {

        try {
            // 获取支付宝POST过来反馈信息
            Map<String, String> params = AliPayApi.toMap(request);

            for (Map.Entry<String, String> entry : params.entrySet()) {
                System.out.println(entry.getKey() + " = " + entry.getValue());
            }

            boolean verifyResult = AlipaySignature.rsaCheckV1(params, parkAliPayProperties.getZfbPublicKey(), "UTF-8", "RSA2");

            if (verifyResult) {

                ParkingRecord parkingRecord = parkingRecordService.findBySerialNumber(params.get("out_trade_no"));
                if (parkingRecord == null) {
                    throw new Exception("支付已完成，但未找到此订单号的流水");
                }
                if (!parkingRecord.getPayAmount().toString().equals(params.get("buyer_pay_amount"))) {
                    throw new Exception("支付金额与系统金额不一致");
                }
                parkingRecord.setUpdateTime(new Date());
                parkingRecord.setUpdateBy(params.get("buyer_logon_id"));
                parkingRecord.setPayType("alipay");
                parkingRecord.setPayStatus("20");
                parkingRecord.setTransactionNumber(params.get("trade_no"));
                parkingRecord.setPaymentTime(DateUtil.parse(params.get("gmt_payment")));

                try {

                    String accessToken = (String) redisService.get("accessToken", "accessToken");
                    if (StringUtils.isBlank(accessToken)){
                        accessToken = parkApiUtil.getXToken();
                    }

                    HashMap<String,Object> map = new HashMap<>();
                    net.sf.json.JSONObject jsonObject = new net.sf.json.JSONObject();
                    if (StringUtils.isBlank(parkingRecord.getParkingDeviceNo())){
                        map.put("carNum",parkingRecord.getCarNum());
                        map.put("consumeMoney",parkingRecord.getPayAmount().floatValue());
                        map.put("feeAmount",parkingRecord.getPayAmount().floatValue());
                  //      jsonObject = parkApiUtil.httpRequest("/ipms/integration/kingdo/payment/success",accessToken,"POST",map);
                    }else {
                        map.put("deviceCode",parkingRecord.getParkingDeviceNo());
                        map.put("feeAmount",parkingRecord.getPayAmount().floatValue());
               //         jsonObject = parkApiUtil.httpRequest("/ipms/payment/bydevice/success",accessToken,"POST",map);

                    }


                    if (!jsonObject.getString("errMsg").equals("success")) {
                        log.error(jsonObject.getString("errMsg"));
                    }






                } catch (Exception e) {
                    log.error(e.getMessage(),e);
                }
                parkingRecordService.update(parkingRecord);


                return "success";
            } else {
                log.error("notify_url 验证失败");
                return "failure";
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage(), e);
            return "failure";
        }
    }


    @GetMapping(value="findUserInfo/{authCode}")
    public MessageResult findUserInfo(@PathVariable(name="authCode") String authCode){

        System.out.println("获取支付宝的userId");
        try{

            AlipayClient alipayClient = new DefaultAlipayClient(parkAliPayProperties.getServiceUrl(),parkAliPayProperties.getAppId(),parkAliPayProperties.getPrivateKey(),parkAliPayProperties.getFormat(),parkAliPayProperties.getInputCharset(),parkAliPayProperties.getZfbPublicKey(),parkAliPayProperties.getSignType());

            AlipaySystemOauthTokenRequest request = new AlipaySystemOauthTokenRequest();
            request.setCode(authCode);
            request.setGrantType("authorization_code");

            AlipaySystemOauthTokenResponse oauthTokenResponse = alipayClient.execute(request);

            System.out.println(oauthTokenResponse.getUserId());

            HashMap<String,Object> dataMap = new HashMap<String,Object>();

            UserInfo userInfo = new UserInfo();
            userInfo.setOpenid(oauthTokenResponse.getUserId());
       //     dataMap.put("openId", oauthTokenResponse.getUserId());

         //   HashMap<String,Object> resultMap = new HashMap<String,Object>();

        //    resultMap.put("userInfo", dataMap);

            return new MessageResult(true, "获取支付宝信息成功", userInfo,200);
        } catch (AlipayApiException e) {



            //处理异常
            e.printStackTrace();
            return new MessageResult(false, "获取支付宝授权失败", "",400);
        } catch (Exception e) {
            //处理异常
            e.printStackTrace();
            return new MessageResult(false, "获取支付宝授权失败", "",400);
        }
    }



}
