package com.jpsoft.smart.schduled;

import com.jpsoft.smart.config.OSSConfig;
import com.jpsoft.smart.modules.base.entity.PersonDeviceLog;
import com.jpsoft.smart.modules.base.service.PersonDeviceFilterLogService;
import com.jpsoft.smart.modules.base.service.PersonDeviceLogService;
import com.jpsoft.smart.modules.common.utils.OSSUtil;
import com.jpsoft.smart.modules.sys.entity.SysLog;
import com.jpsoft.smart.modules.sys.service.SysLogService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import java.util.Date;
import java.util.List;

@Component
@Slf4j
public class CleanVisitorRecordTask {

    @Autowired
    private PersonDeviceFilterLogService personDeviceFilterLogService;

    @Autowired
    private PersonDeviceLogService personDeviceLogService;

    @Autowired
    private SysLogService sysLogService;

    @Autowired
    private OSSConfig ossConfig;

    //每1小时执行一次
    @Scheduled(cron="0 0 0/1 * * ?")
    public void run(){
        List<PersonDeviceLog> logList = personDeviceLogService.findExpiredVisitorRecord(100);

        int removeImageCount = 0;
        int removeRawCount = 0;
        int removeFilterCount = 0;

        //删除上传图片
        for (PersonDeviceLog personDeviceLog : logList) {
            String faceImage = personDeviceLog.getFaceImage();

            if(OSSUtil.deleteFile(ossConfig,faceImage)){
                removeImageCount++;
            }

            removeRawCount += personDeviceLogService.delete(personDeviceLog.getId());
        }

        //删除过滤记录
        removeFilterCount = personDeviceFilterLogService.deleteExpiredVisitorRecord(100);

        //增加系统日志
        SysLog sysLog = new SysLog();
        sysLog.setPointcut("清除过期访问者测温记录");
        sysLog.setRemark(String.format("删除图片数量:%s,删除原始测温记录:%s,删除过滤后测温记录:%s",
                removeImageCount,removeRawCount,removeFilterCount));
        sysLog.setCreateTime(new Date());

        sysLogService.insert(sysLog);
    }
}
