package com.jpsoft.smart.schduled;

import com.jpsoft.smart.modules.base.entity.DeviceInfo;
import com.jpsoft.smart.modules.base.service.DeviceInfoService;
import com.jpsoft.smart.modules.lapi.service.ILapiService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

/**
 * @author 墨鱼_mo
 * @date 2020-3-25 11:11
 */
@Component
@Slf4j
public class HeartbeatDetectionTask {

    @Autowired
    private DeviceInfoService deviceInfoService;

    @Autowired
    private ILapiService lapiService;

    //每2分钟执行一次
    @Scheduled(cron = "0 0/2 * * * ?")
    public void run() {

        log.warn("设备在线定时任务启动");
        List<DeviceInfo> deviceInfoList = deviceInfoService.listOnline();

        if (deviceInfoList.size() > 0) {

            for (DeviceInfo deviceInfo : deviceInfoList) {
                if (deviceInfo.getHeartbeatUpdateTime() != null) {
                    Long heartbeatUpdateTime = deviceInfo.getHeartbeatUpdateTime().getTime();
                    if (new Date().getTime() - heartbeatUpdateTime > 2 * 60 * 1000) {
                        deviceInfo.setIsOnline(false);
                        try {
                            Boolean isKeepAlive = lapiService.keepAlive(deviceInfo.getId());
                            if (isKeepAlive) {
                                deviceInfo.setAbnormalReason("设备可正常访问，但无心跳返回，请查看设备服务器配置");
                            } else {
                                deviceInfo.setAbnormalReason("设备离线");
                            }
                            deviceInfoService.update(deviceInfo);
                        } catch (Exception e) {
                            deviceInfo.setAbnormalReason("设备离线");
                            deviceInfoService.update(deviceInfo);
                        }

                    }
                }
            }

        }

    }


}
