package com.jpsoft.smart.schduled;

import cn.hutool.core.date.DateUtil;
import com.jpsoft.smart.config.OSSConfig;
import com.jpsoft.smart.config.WxConfig;
import com.jpsoft.smart.modules.base.entity.*;
import com.jpsoft.smart.modules.base.service.*;
import com.jpsoft.smart.modules.business.entity.WorkAttendance;
import com.jpsoft.smart.modules.business.service.WorkAttendanceService;
import com.jpsoft.smart.modules.common.utils.JwtUtil;
import com.jpsoft.smart.modules.common.utils.OSSUtil;
import com.jpsoft.smart.modules.common.utils.POIUtils;
import com.jpsoft.smart.modules.common.utils.WechatMessageUtil;
import com.sun.corba.se.spi.orbutil.threadpool.Work;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.security.Keys;
import lombok.extern.slf4j.Slf4j;
import net.sf.json.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.joda.time.DateTime;
import org.joda.time.Days;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.ClassPathResource;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.net.URLEncoder;
import java.security.Key;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@Component
@Slf4j
public class UnmeasureTemperatureAlarmTask {
    @Value("${jwt.secret}")
    private String jwtSecret;

    @Autowired
    private AlarmConfigService alarmConfigService;

    @Autowired
    private WarningPusherService warningPusherService;

    @Autowired
    private PersonDeviceFilterLogService personDeviceFilterLogService;

    @Autowired
    private CompanyInfoService companyInfoService;

    @Autowired
    private CompanyPositionService companyPositionService;

    @Autowired
    private WorkAttendanceService workAttendanceService;

    @Autowired
    private PersonDeviceLogService personDeviceLogService;

    @Autowired
    private PersonInfoService personInfoService;

    @Autowired
    private WxConfig wxConfig;

    @Autowired
    private OSSConfig ossConfig;

    @Value("${mobile.unMeasureUrl}")
    private String unMeasureUrl;

    @Value("${wx.notice.remindTmplCode}")
    private String remindTmplCode;

    @Value("${wx.notice.supervisionTmplCode}")
    private String supervisionTmplCode;

    @Autowired
    private HolidayInfoService holidayInfoService;

    public int batchUpdate(String companyId,Date startDate,Date endDate) throws Exception {
        DateTime dt1 = new DateTime(startDate);
        DateTime dt2 = new DateTime(endDate);

        int days = Days.daysBetween(dt1,dt2).getDays();

        List<AlarmConfig> configList = alarmConfigService.findByCompanyId(companyId);

        CompanyInfo companyInfo = companyInfoService.get(companyId);

        workAttendanceService.deleteByCompanyCodeAndDate(companyInfo.getCode() + "%",startDate,endDate);

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");

        int affectCount = 0;

        //更新时间段内节假日
        Set<String> holidaySet = holidayInfoService.getHolidaySetByDate(startDate,endDate);

        for (int i=0;i<days;i++){
            DateTime now = dt1.plusDays(i);

            String date = now.toString("yyyy-MM-dd");

            if (holidaySet.contains(date)){
                continue;
            }

            for (AlarmConfig alarmConfig : configList) {
                //是否工作日
                if (alarmConfig.getWeekdays().indexOf(String.valueOf(now.getDayOfWeek())) == -1) {
                    continue;
                }

                DateTime startTime = new DateTime(sdf.parse(date + " " + alarmConfig.getStartTime()));
                DateTime attendanceTime = new DateTime(sdf.parse(date + " " + alarmConfig.getAttendanceTime()));
                DateTime endTime = new DateTime(sdf.parse(date + " " + alarmConfig.getEndTime()));

                String companyCode = companyInfo.getCode();

                if (alarmConfig.getSubordinate()!=null && alarmConfig.getSubordinate()){
                    companyCode += "%";
                }

                //todo 查询该单位当前时间段考勤人员列表
                List<PersonInfo> attendanceList = personDeviceLogService.queryAttendanceList(companyCode,startTime.toDate(),endTime.toDate());

                for (PersonInfo personInfo: attendanceList) {
                    WorkAttendance workAttendance = new WorkAttendance();
                    workAttendance.setId(UUID.randomUUID().toString());
                    workAttendance.setPersonId(personInfo.getId());
                    workAttendance.setAlarmConfigId(alarmConfig.getId());
                    workAttendance.setCreateTime(new Date());
                    workAttendance.setDelFlag(false);

                    List<PersonDeviceLog> logList = personDeviceLogService.findByPersonAndDate(personInfo.getId(),startTime.toDate(),endTime.toDate());

                    if (logList.size()>0) {
                        PersonDeviceLog personDeviceLog = null;

                        if (alarmConfig.getClassifier().equals(1)) {
                            //上班算最早一次考勤
                            personDeviceLog = logList.get(0);

                            if(personDeviceLog.getRecordTime().compareTo(startTime.toDate())>0 && personDeviceLog.getRecordTime().compareTo(attendanceTime.toDate())<=0){
                                workAttendance.setResult(WorkAttendance.SUCCESS);
                            }
                            else if(personDeviceLog.getRecordTime().compareTo(attendanceTime.toDate())>0 && personDeviceLog.getRecordTime().compareTo(endTime.toDate())<=0){
                                workAttendance.setResult(WorkAttendance.LATE);
                            }
                        } else {
                            //下班算最迟一次考勤
                            personDeviceLog = logList.get(logList.size()-1);

                            if(personDeviceLog.getRecordTime().compareTo(startTime.toDate())>0 && personDeviceLog.getRecordTime().compareTo(attendanceTime.toDate())<=0){
                                workAttendance.setResult(WorkAttendance.LEAVE_EARLY);
                            }
                            else if(personDeviceLog.getRecordTime().compareTo(attendanceTime.toDate())>0 && personDeviceLog.getRecordTime().compareTo(endTime.toDate())<=0){
                                workAttendance.setResult(WorkAttendance.SUCCESS);
                            }
                        }

                        workAttendance.setRecordTime(personDeviceLog.getRecordTime());

                        workAttendance.setTemperature(personDeviceLog.getTemperature());
                    }

                    affectCount +=  workAttendanceService.insert(workAttendance);
                }

                //统计缺卡
                recordUnAttendance(alarmConfig,startTime.toDate(),attendanceTime.toDate(),endTime.toDate(),WorkAttendance.MISSING);
            }
        }

        return affectCount;
    }

    //每5分钟执行一次
    @Scheduled(cron="0 0/5 * * * ?")
    public void run() {
        int intervalMinute = 5;

        List<AlarmConfig> configList = alarmConfigService.list();

        DateTime now = DateTime.now();

        String date = now.toString("yyyy-MM-dd");

        //最近一年的节假日
        Set<String> holidaySet = holidayInfoService.getHolidaySetByDate(now.minusDays(364).toDate(),now.plusDays(1).toDate());

        if (holidaySet.contains(date)){
            return;
        }

        int weekday = now.getDayOfWeek();

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");

        for (AlarmConfig alarmConfig : configList) {
            try {
                String weekdays = alarmConfig.getWeekdays();
                DateTime startTime = new DateTime(sdf.parse(date + " " + alarmConfig.getStartTime()));
                DateTime attendanceTime = new DateTime(sdf.parse(date + " " + alarmConfig.getAttendanceTime()));
                DateTime endTime = new DateTime(sdf.parse(date + " " + alarmConfig.getEndTime()));

                DateTime alarmTime = null;

                if (alarmConfig.getClassifier()!=null && alarmConfig.getClassifier().equals(1)){
                    //todo 上班考勤时间提前10分钟通知未打卡人员
                    alarmTime = attendanceTime.plusMinutes(-10);
                }
                else{
                    //todo 下班考勤时间延迟10分钟通知未打卡人员
                    alarmTime = attendanceTime.plusMinutes(10);
                }

                //开始时间
                if (weekdays.indexOf(String.valueOf(weekday)) != -1) {
                    if (now.compareTo(alarmTime)>=0 && now.compareTo(alarmTime.plusMinutes(intervalMinute))<0) {
                        //todo 通知个人
                        if(alarmConfig.getNeedMeasureTemperature()){
                            noticePerson(alarmConfig,startTime,endTime);
                        }
                    }

                    //todo 考勤时间已到
                    if (now.compareTo(attendanceTime)>=0 && now.compareTo(attendanceTime.plusMinutes(intervalMinute))<0) {
                        //todo  未测量人通知管理人员
                        if(alarmConfig.getClassifier().equals(1)){
                            //上班
                            if(alarmConfig.getNeedMeasureTemperature()) {
                                noticeManager(alarmConfig, startTime, attendanceTime);
                            }
                        }

                        //todo 记录考勤
                        if (alarmConfig.getClassifier().equals(1)) {
                            //上班时间前打卡算正常上班
//                            recordAttendance(alarmConfig, startTime.toDate(), attendanceTime.toDate(), WorkAttendance.SUCCESS);
                        }
                        else{
                            //下班时间前打卡算早退
//                            recordAttendance(alarmConfig, startTime.toDate(), attendanceTime.toDate(), WorkAttendance.LEAVE_EARLY);
                        }
                    }

                    //todo 截止时间已到
                    if (now.compareTo(endTime)>=0 && now.compareTo(endTime.plusMinutes(intervalMinute))<0) {
                        if(alarmConfig.getClassifier().equals(2)){
                            //下班
                            if(alarmConfig.getNeedMeasureTemperature()) {
                                noticeManager(alarmConfig, attendanceTime, endTime);
                            }
                        }

                        //todo 记录考勤
                        if (alarmConfig.getClassifier().equals(1)) {
                            //上班时间后截止时间前打卡算迟到
//                            recordAttendance(alarmConfig, attendanceTime.toDate(), endTime.toDate(),WorkAttendance.LATE);
                        }
                        else{
                            //下班时间后截止时间前打卡算正常下班
//                            recordAttendance(alarmConfig, attendanceTime.toDate(), endTime.toDate(),WorkAttendance.SUCCESS);
                        }

                        //未在开始截止时间内打卡算旷工
                        recordUnAttendance(alarmConfig,startTime.toDate(),attendanceTime.toDate(),endTime.toDate(),WorkAttendance.MISSING);
                    }
                }
            }
            catch (Exception ex){
                log.error(ex.getMessage(),ex);
            }
        }
    }

    /**
     * 记录未考勤人员
     * @param alarmConfig
     * @param startDate
     * @param endDate
     * @param result
     */
    private int recordUnAttendance(AlarmConfig alarmConfig, Date startDate,Date attendanceDate, Date endDate, String result) {
        int affectCount = 0;

        CompanyInfo companyInfo = companyInfoService.get(alarmConfig.getCompanyId());

        String companyCode = companyInfo.getCode();

        if (alarmConfig.getSubordinate() != null && alarmConfig.getSubordinate()) {
            companyCode += "%";
        }

        //todo 查询该单位当前时间段未考勤人员列表
        List<PersonInfo> attendanceList = personDeviceLogService.queryUnAttendanceList(companyCode, startDate, endDate);

        for (PersonInfo personInfo : attendanceList) {
            WorkAttendance workAttendance = new WorkAttendance();
            workAttendance.setId(UUID.randomUUID().toString());
            workAttendance.setPersonId(personInfo.getId());
            workAttendance.setAlarmConfigId(alarmConfig.getId());
            workAttendance.setCreateTime(new Date());
            workAttendance.setRecordTime(attendanceDate);
            workAttendance.setResult(result);
            workAttendance.setDelFlag(false);

            affectCount+= workAttendanceService.insert(workAttendance);
        }

        return affectCount;
    }

    /**
     * 记录考勤
     * @param alarmConfig
     */
    private int recordAttendance(AlarmConfig alarmConfig,Date startDate,Date endDate,String result) {
        CompanyInfo companyInfo = companyInfoService.get(alarmConfig.getCompanyId());

        String companyCode = companyInfo.getCode();

        if (alarmConfig.getSubordinate()!=null && alarmConfig.getSubordinate()){
            companyCode += "%";
        }

        int affectCount = 0;

        //todo 查询该单位当前时间段考勤人员列表
        List<PersonInfo> attendanceList = personDeviceLogService.queryAttendanceList(companyCode,startDate,endDate);

        for (PersonInfo personInfo: attendanceList) {
            WorkAttendance workAttendance = new WorkAttendance();
            workAttendance.setId(UUID.randomUUID().toString());
            workAttendance.setPersonId(personInfo.getId());
            workAttendance.setAlarmConfigId(alarmConfig.getId());
            workAttendance.setCreateTime(new Date());
            workAttendance.setResult(result);
            workAttendance.setDelFlag(false);

            List<PersonDeviceLog> logList = personDeviceLogService.findByPersonAndDate(personInfo.getId(),startDate,endDate);

            if (logList.size()>0) {
                PersonDeviceLog personDeviceLog = null;

                if (alarmConfig.getClassifier().equals(1)) {
                    //上班算最早一次考勤
                    personDeviceLog = logList.get(0);
                } else {
                    //下班算最迟一次考勤
                    personDeviceLog = logList.get(logList.size()-1);
                }

                workAttendance.setRecordTime(personDeviceLog.getRecordTime());
                workAttendance.setTemperature(personDeviceLog.getTemperature());
            }

            affectCount +=  workAttendanceService.insert(workAttendance);
        }

        return affectCount;
    }

    @Async
    public void noticePerson(AlarmConfig alarmConfig, DateTime startTime, DateTime endTime) {
        CompanyInfo companyInfo = companyInfoService.get(alarmConfig.getCompanyId());

        String companyCode = companyInfo.getCode();

        if (alarmConfig.getSubordinate()!=null && alarmConfig.getSubordinate()){
            companyCode += "%";
        }

        //todo 查询该单位当前时间段是否有人员未测体温
        List<PersonInfo> unmeasuredList = personDeviceLogService.queryUnAttendanceList(companyCode,startTime.toDate(),endTime.toDate());

        if (unmeasuredList.size()==0){
            return;
        }

        //todo 给未测量人推送微信通知
        for (PersonInfo personInfo : unmeasuredList) {
            if (personInfo.getWechatNoticeEnabled()!=null
            && personInfo.getWechatNoticeEnabled()
            && StringUtils.isNotEmpty(personInfo.getOpenId())) {
                String message = "尊敬的" + personInfo.getName();

//                if (alarmConfig.getClassifier()!=null && alarmConfig.getClassifier().equals(1)){
//                    message += ",请在" + startTime.toString("HH:mm") + "至" + attendanceTime.toString("HH:mm") + "时间内在单位（校园）内任意打卡点打卡及测温!";
//                }
//                else{
//                    message += ",请在" + attendanceTime.toString("HH:mm") + "至" + endTime.toString("HH:mm") + "时间内在单位（校园）内任意打卡点打卡及测温!";
//                }
                message +=",请抓紧时间在单位（校园）内任意打卡点打卡和测温!";

                JSONObject sendData = new JSONObject();

                JSONObject first = new JSONObject();
                first.put("value", "未测温提醒");
                first.put("color", "#FF0000");

                JSONObject keyword1 = new JSONObject();
                keyword1.put("value", personInfo.getName());
                keyword1.put("color", "#173177");

                JSONObject keyword2 = new JSONObject();
                keyword2.put("value", DateUtil.format(new Date(), "yyyy-MM-dd HH:mm"));
                keyword2.put("color", "#173177");

                JSONObject keyword3 = new JSONObject();
                keyword3.put("value", "待测温");
                keyword3.put("color", "#173177");

                JSONObject remark = new JSONObject();
                remark.put("value", message);
                remark.put("color", "#173177");

                sendData.put("first", first);
                sendData.put("keyword1", keyword1);
                sendData.put("keyword2", keyword2);
                sendData.put("keyword3", keyword3);
                sendData.put("remark", remark);

                WechatMessageUtil.sendTemplate(sendData, wxConfig.getAppId(),wxConfig.getAppSecret(),remindTmplCode,personInfo.getOpenId(), null);
            }
        }
    }

    @Async
    public void noticeManager(AlarmConfig alarmConfig, DateTime startTime, DateTime endTime) {
        CompanyInfo companyInfo = companyInfoService.get(alarmConfig.getCompanyId());

        String companyCode = companyInfo.getCode();

        if (alarmConfig.getSubordinate()!=null && alarmConfig.getSubordinate()){
            companyCode += "%";
        }

        //todo 查询该单位当前时间段是否有人员未测体温
        List<PersonInfo> unmeasuredList = personDeviceLogService.queryUnAttendanceList(companyCode,startTime.toDate(),endTime.toDate());

        if (unmeasuredList.size()==0){
            return;
        }

        int total = unmeasuredList.size();

        //todo 查询该单位的通知人
        List<WarningPusher> pusherList = warningPusherService.findByCompanyId(companyInfo.getId());

        //todo 给单位相关人员推送通知
        for (WarningPusher pusher: pusherList) {
            if (StringUtils.isNotEmpty(pusher.getOpenId())
            && pusher.getMissingNoticeEnabled()!=null && pusher.getMissingNoticeEnabled()) {
                String message = "尊敬的" + pusher.getName()
                        + ",当天时段" + startTime.toString("HH:mm") + "至" + endTime.toString("HH:mm")
                        + ",有" + total + "人未测量体温，具体名单请点击下方【查看详情】查看";

                //todo 生成未测量人员名单
                PersonInfo personInfo = personInfoService.findByOpenId(pusher.getOpenId());
                long personId = 0;

                if (personInfo!=null){
                    personId = personInfo.getId();
                }

                try {
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

                    String token = JwtUtil.createToken(jwtSecret,String.valueOf(personId),sdf.parse("2030-01-01"));

                    String url = unMeasureUrl + "?token=" + URLEncoder.encode(token,"UTF-8");
                    url += "&companyId=" + alarmConfig.getCompanyId();
                    url += "&queryDate=" + startTime.toString("yyyy-MM-dd");
                    url += "&alarmConfigId=" + alarmConfig.getId();

                    JSONObject sendData = new JSONObject();

                    JSONObject first = new JSONObject();
                    first.put("value", "未测温提醒");
                    first.put("color", "#FF0000");

                    JSONObject keyword1 = new JSONObject();
                    keyword1.put("value", DateUtil.format(new Date(), "yyyy-MM-dd HH:mm"));
                    keyword1.put("color", "#173177");

                    JSONObject keyword2 = new JSONObject();
                    keyword2.put("value", companyInfo.getName());
                    keyword2.put("color", "#173177");

                    JSONObject keyword3 = new JSONObject();
                    keyword3.put("value", pusher.getName());
                    keyword3.put("color", "#173177");

                    JSONObject remark = new JSONObject();
                    remark.put("value", message);
                    remark.put("color", "#173177");

                    sendData.put("first", first);
                    sendData.put("keyword1", keyword1);
                    sendData.put("keyword2", keyword2);
                    sendData.put("keyword3", keyword3);
                    sendData.put("remark", remark);

                    WechatMessageUtil.sendTemplate(sendData, wxConfig.getAppId(),wxConfig.getAppSecret(),supervisionTmplCode,pusher.getOpenId(), url);
                }
                catch(Exception ex){
                    log.error(ex.getMessage(),ex);
                }
            }
        }
    }

    private String createUnMeasureReport(CompanyInfo companyInfo, List<PersonInfo> unmeasuredList, DateTime startTime, DateTime endTime) throws Exception{
        ClassPathResource resource = new ClassPathResource("static/unMeasureReport.xls");

        Workbook wb = WorkbookFactory.create(resource.getInputStream());

        Sheet sheet = wb.getSheetAt(0);

        Row row1 = sheet.getRow(0);
        Row row2 = sheet.getRow(1);
        Row row3 = sheet.getRow(2);

        String title = row1.getCell(0).getStringCellValue();
        String subTitle = row2.getCell(0).getStringCellValue();

        row1.getCell(0).setCellValue(String.format(title, startTime.toString("HH:mm"), endTime.toString("HH:mm")));
        row2.getCell(0).setCellValue(String.format(subTitle, DateTime.now().toString("yyyy-MM-dd HH:mm")));

        //显示扩展字段
        List<CompanyPosition> companyPositions = companyPositionService.findByCompanyId(companyInfo.getId());

        if (companyPositions.size()>0){
            CompanyPosition companyPosition = companyPositions.get(0);

            if(StringUtils.isNotEmpty(companyPosition.getPosition1Name())) {
                row3.createCell(3).setCellValue(companyPosition.getPosition1Name());
            }

            if(StringUtils.isNotEmpty(companyPosition.getPosition2Name())) {
                row3.createCell(4).setCellValue(companyPosition.getPosition2Name());
            }

            if(StringUtils.isNotEmpty(companyPosition.getPosition3Name())) {
                row3.createCell(5).setCellValue(companyPosition.getPosition3Name());
            }

            if(StringUtils.isNotEmpty(companyPosition.getPosition4Name())) {
                row3.createCell(6).setCellValue(companyPosition.getPosition4Name());
            }

            if(StringUtils.isNotEmpty(companyPosition.getPosition5Name())) {
                row3.createCell(7).setCellValue(companyPosition.getPosition5Name());
            }
        }

        int rowIndex = 3;

        for (PersonInfo personInfo : unmeasuredList) {
            Row row = sheet.createRow(rowIndex);
            row.createCell(0).setCellValue(rowIndex-2);
            row.createCell(1).setCellValue(personInfo.getCompanyName());
            row.createCell(2).setCellValue(personInfo.getName());
            row.createCell(3).setCellValue(personInfo.getPosition1());
            row.createCell(4).setCellValue(personInfo.getPosition2());
            row.createCell(5).setCellValue(personInfo.getPosition3());
            row.createCell(6).setCellValue(personInfo.getPosition4());
            row.createCell(7).setCellValue(personInfo.getPosition4());

            rowIndex++;
        }

        //todo 将wb保存到oss
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        wb.write(output);
        byte[] buffer = output.toByteArray();

        ByteArrayInputStream input = new ByteArrayInputStream(buffer);

        String fileName = "未测温统计表.xls";

        String downloadUrl = OSSUtil.upload(ossConfig, "unmeasuredReport", fileName, input);

        wb.write(output);
        wb.close();
        output.close();

        return downloadUrl;
    }

    public static void main(String[] args) {
        Boolean result = null;

        if (result){
            System.out.println("yes");
        }
        else{
            System.out.println("no");
        }
    }
}
